@@ -24,6 +24,7 @@
 ** cmesh.cpp
 */
 
+
 #include "cmesh.h" 
 #include "view.h" 
 
@@ -31,7 +32,7 @@
 
 
 cmesh::cmesh(base *p,char *n,texture *t) :
-	csgobj(p,n,t)
+	smesh(p,n,t)
 {
 	numtype = NUM_CMESH;
 
@@ -43,33 +44,39 @@ cmesh::cmesh(base *p,char *n,texture *t) :
 
 
 cmesh::cmesh(base *p,char *n,Vector3 &v1,Vector3 &v2,Vector3 &v3):
-	csgobj(p,n)
+	smesh(p,n)
 {
+	numtype = NUM_CMESH;
+
 	if(parent)
 		addToParent(parent);
 
 	gptr = 0;
 
-	addSegment(addKnode(v1),addKnode(v2),addKnode(v3));
+	addSegment(addNode(v1),addNode(v2),addNode(v3));
 }
 
 cmesh::cmesh(base *p,char *n,texture *t,Vector3 &v1,Vector3 &v2,Vector3 &v3) :
-	csgobj(p,n,t)
+	smesh(p,n,t)
 {
+	numtype = NUM_CMESH;
+
 	if(parent)
 		addToParent(parent);
 
 	gptr = 0;
 
-	addSegment(addKnode(v1),addKnode(v2),addKnode(v3));
+	addSegment(addNode(v1),addNode(v2),addNode(v3));
 }
 
 cmesh::cmesh(base *p,cmesh *mc) :
-	csgobj(p,mc)
+	smesh(p,mc)
 {
 	int		i,s0,s1,s2;
 	Vector3		v0,v1;
 
+	numtype = NUM_CMESH;
+
 	gptr = 0;
 
 	if(parent)
@@ -77,8 +84,8 @@ cmesh::cmesh(base *p,cmesh *mc) :
 
 	for(i = 0;i < mc->vectorlist.length();i++)
 	{
-		if(mc->getKnode(i,v0,v1) >= 0)
-			addKnode(v0,v1);
+		if(mc->getNode(i,v0,v1) >= 0)
+			addNode(v0,v1);
 	}
 	for(i = 0;i < mc->trianglelist.length();i++)
 	{
@@ -87,9 +94,11 @@ cmesh::cmesh(base *p,cmesh *mc) :
 	}
 }
 
-cmesh::cmesh(mesh &m) : 	// Typumwandlung
-	csgobj(m.getParent(),"Unnamed")
+cmesh::cmesh(smesh &m) : 	// Typumwandlung
+	smesh(m.getParent(),"Unnamed")
 {
+	numtype = NUM_CMESH;
+
 	gptr = 0;
 
 	printf("Typecast not implemented\n");
@@ -107,123 +116,6 @@ base		*cmesh::copy(base *p)
 	return new cmesh(p,this);
 }
 
-int		cmesh::addKnode(double x, double y, double z,double nx, double ny, double nz)
-{
-	return addKnode(Vector3(x,y,z),Vector3(nx,ny,nz));
-}
-
-int		cmesh::addKnode(Vector3 v,Vector3 n)
-{
-	dvec		dv;
-	int		i;
-
-	dv.position= v;
-	dv.direction = n;
-
-	if((i = vectorlist.append(dv)) != 0)
-		return i;
-
-	if(gptr)
-	{
-		delete gptr;
-		gptr = 0;
-	}
-
-	return vectorlist.length() - 1;
-}
-
-int	cmesh::addSegment(int p1,int p2,int p3)
-{
-	seg		s;
-	int		i;
-
-	if(p1 < 0 || p1 >= vectorlist.length())
-		return -1; // p1 out of range
-	if(p2 < 0 || p2 >= vectorlist.length())
-		return -2; // p2 out of range
-	if(p3 < 0 || p3 >= vectorlist.length())
-		return -3; // p3 out of range
-
-	s.pointnum[0] = p1;
-	s.pointnum[1] = p2;
-	s.pointnum[2] = p3;
-
-	if((i = trianglelist.append(s)) != 0)
-		return i;
-
-	if(gptr)
-	{
-		delete gptr;
-		gptr = 0;
-	}
-
-	return trianglelist.length() - 1;
-}
-
-int		cmesh::getKnode(int n,Vector3 &v,Vector3 &nv)
-{
-	if(n < 0 || n > vectorlist.length())
-		return -1;
-
-	v = vectorlist[n].direction;
-	nv = vectorlist[n].position;
-
-	return 0;
-}
-
-Vector3		&cmesh::getPoint(int n)
-{
-	return vectorlist[n].position;
-}
-
-Vector3		&cmesh::getNormal(int n)
-{
-	return vectorlist[n].direction;
-}
-
-int		cmesh::getSegment(int n,int &s0,int &s1,int &s2)
-{
-	if(n < 0 || n > trianglelist.length()) 
-		return  -1;
-
-	s0 = trianglelist[n].pointnum[0]; 
-	s1 = trianglelist[n].pointnum[1]; 
-	s2 = trianglelist[n].pointnum[2];
-
-	return 0;
-}
-
-int		cmesh::removeSegment(int segnum)
-{
-	if(gptr)
-	{
-		delete gptr;
-		gptr = 0;
-	}
-
-	return trianglelist.deleteAt(segnum);
-}
-
-int		cmesh::removeKnode(int knodenum)
-{
-	int	i,s[3];
-
-	if(gptr)
-	{
-		delete gptr;
-		gptr = 0;
-	}
-
-	for(i = 0;i < trianglelist.length();i++)
-	{
-		getSegment(i,s[0],s[1],s[2]);
-
-		if(knodenum == s[0] || knodenum == s[1] || knodenum == s[2])
-			return -1;
-	}
-
-	return vectorlist.deleteAt(knodenum);
-}
 
 
 graph		*cmesh::createGraph()
@@ -236,30 +128,32 @@ graph		*cmesh::createGraph()
 	gptr = new graph();
 	for(i = 0;i < vectorlist.length();i++)
 	{
-		new knode(gptr,vectorlist[i].position);
+		new node(gptr,vectorlist[i].position);
 	}
 	for(i = 0;i < trianglelist.length();i++)
 	{
 		s1 = trianglelist[i].pointnum[0];
 		s2 = trianglelist[i].pointnum[1];
 		s3 = trianglelist[i].pointnum[2];
-		new edge(gptr,gptr->getKnode(s1),gptr->getKnode(s2),gptr->numEdges() + 1);
-		new edge(gptr,gptr->getKnode(s2),gptr->getKnode(s3),gptr->numEdges() + 1);
-		new edge(gptr,gptr->getKnode(s3),gptr->getKnode(s1),gptr->numEdges() + 1);
+		new edge(gptr,gptr->getNode(s1),gptr->getNode(s2),gptr->numEdges() + 1);
+		new edge(gptr,gptr->getNode(s2),gptr->getNode(s3),gptr->numEdges() + 1);
+		new edge(gptr,gptr->getNode(s3),gptr->getNode(s1),gptr->numEdges() + 1);
 	}
 
 	return gptr;
 }
 
 
+
 int		cmesh::createFromGraph(graph *g)
 {
-	int		i,t,c;
+	int		i,t,c,l,s1,s2,s3;
 	Vector3		v;
 	list<edge*>	del;
 	edge		*e1,*e2,*e3;
-	knode		*k;
+	node		*k;
 	graph		*gt;
+	bool		sf;
 
 	if(!g)
 		return 0;
@@ -270,117 +164,158 @@ int		cmesh::createFromGraph(graph *g)
 	gt = gptr;
 	gptr = 0;
 
-	for(i = 0;i < g->numKnodes();i++)
+	// take all the points
+	for(i = 0;i < g->numNodes();i++)
 	{
-		k = g->getKnode(i);
+		k = g->getNode(i);
 		v = k->vector();
-		addKnode(v);
+		addNode(v);
 	}
+	
+	// fill the del (double edge list) with every edge twice
+	// in a close mesh every edge is part of exactly 2 triangles
 	for(i = 0;i < g->numEdges();i++)
 	{
 		del += g->getEdge(i);
 	}
-	for(i = 0;i < g->numEdges();i++)
+	for(i = g->numEdges() - 1;i >= 0;i--)
 	{
 		del += g->getEdge(i);
 	}
 
-	for(c = 0;c < del.length();c++)
+	l = del.length();
+	printf("del len %i\n",l);
+	// create the segments
+	for(c = 0;del.length() > 0 && c < l * 2;c++)
 	{
+		printf("%i. ",c);
+		sf = false;
 		e1 = del[c % del.length()];
-		for(i = 0;i < e1->knode2()->numEdges();i++)
+		for(i = 0;!sf && i < e1->node2()->numEdges();i++)
 		{
-			e2 = e1->knode2()->getEdge(i);
-			if(e1 == e2 || del.find(e2) < 0 || e1->knode2() != e2->knode1())
+			e2 = e1->node2()->getEdge(i);
+			if(e1 == e2 || del.find(e2) < 0 || e1->node2() != e2->node1())
 				continue;
-			for(t = 0;t < e2->knode2()->numEdges();t++)
+			for(t = 0;!sf && t < e2->node2()->numEdges();t++)
 			{
-				e3 = e2->knode2()->getEdge(t);
+				e3 = e2->node2()->getEdge(t);
 				if(e1 == e3 || e2 == e3 || del.find(e3) < 0)
 					continue;
 /*
 				printf("seg(%i,%i,%i) <%g,%g,%g><%g,%g,%g><%g,%g,%g>\n",e1->id(),e2->id(),e3->id(),
-					e1->knode1()->vector()[0],e1->knode1()->vector()[1],e1->knode1()->vector()[2],
-					e2->knode1()->vector()[0],e2->knode1()->vector()[1],e2->knode1()->vector()[2],
-					e3->knode1()->vector()[0],e3->knode1()->vector()[1],e3->knode1()->vector()[2]);
+					e1->node1()->vector()[0],e1->node1()->vector()[1],e1->node1()->vector()[2],
+					e2->node1()->vector()[0],e2->node1()->vector()[1],e2->node1()->vector()[2],
+					e3->node1()->vector()[0],e3->node1()->vector()[1],e3->node1()->vector()[2]);
 */
-				if(e2->knode2() != e3->knode1() || e3->knode2() != e1->knode1())
+				if(e2->node2() != e3->node1() || e3->node2() != e1->node1())
 					continue;
-				if(addSegment(e1,e2,e3))
+				s1 = getNodeIndex(e1->node1());
+				s2 = getNodeIndex(e2->node1());
+				s3 = getNodeIndex(e3->node1());
+				if(!hasSegment(s1,s2,s3))
 				{
-					del.deleteAt(del.find(e1));
-					del.deleteAt(del.find(e2));
-					del.deleteAt(del.find(e3));
-					e1 = del[c % del.length()];
+					if(addSegment(s1,s2,s3))
+					{
+						del.deleteAt(del.find(e1));
+						del.deleteAt(del.find(e2));
+						del.deleteAt(del.find(e3));
+//						e1 = del[c % del.length()];
+						sf = true;
+						break;
+					}
 				}
 			}
-			for(t = 0;t < e2->knode2()->numEdges();t++)
+			for(t = 0;!sf && t < e2->node2()->numEdges();t++)
 			{
-				e3 = e2->knode2()->getEdge(t)->swap();
+				e3 = e2->node2()->getEdge(t)->swap();
 				if(e1 == e3 || e2 == e3 || del.find(e3) < 0)
 					continue;
 /*
 				printf("seg(%i,%i,%i) <%g,%g,%g><%g,%g,%g><%g,%g,%g>\n",e1->id(),e2->id(),e3->id(),
-					e1->knode1()->vector()[0],e1->knode1()->vector()[1],e1->knode1()->vector()[2],
-					e2->knode1()->vector()[0],e2->knode1()->vector()[1],e2->knode1()->vector()[2],
-					e3->knode1()->vector()[0],e3->knode1()->vector()[1],e3->knode1()->vector()[2]);
+					e1->node1()->vector()[0],e1->node1()->vector()[1],e1->node1()->vector()[2],
+					e2->node1()->vector()[0],e2->node1()->vector()[1],e2->node1()->vector()[2],
+					e3->node1()->vector()[0],e3->node1()->vector()[1],e3->node1()->vector()[2]);
 */
-				if(e2->knode2() != e3->knode1() || e3->knode2() != e1->knode1())
+				if(e2->node2() != e3->node1() || e3->node2() != e1->node1())
 					continue;
-				if(addSegment(e1,e2,e3))
+				s1 = getNodeIndex(e1->node1());
+				s2 = getNodeIndex(e2->node1());
+				s3 = getNodeIndex(e3->node1());
+				if(!hasSegment(s1,s2,s3))
 				{
-					del.deleteAt(del.find(e1));
-					del.deleteAt(del.find(e2));
-					del.deleteAt(del.find(e3));
-					e1 = del[c % del.length()];
+					if(addSegment(s1,s2,s3))
+					{
+						del.deleteAt(del.find(e1));
+						del.deleteAt(del.find(e2));
+						del.deleteAt(del.find(e3));
+//						e1 = del[c % del.length()];
+						sf = true;
+						break;
+					}
 				}
 			}
 		}
-		for(i = 0;i < e1->knode2()->numEdges();i++)
+		for(i = 0;!sf && i < e1->node2()->numEdges();i++)
 		{
-			e2 = e1->knode2()->getEdge(i)->swap();
-			if(e1 == e2 || del.find(e2) < 0 || e1->knode2() != e2->knode1())
+			e2 = e1->node2()->getEdge(i)->swap();
+			if(e1 == e2 || del.find(e2) < 0 || e1->node2() != e2->node1())
 				continue;
-			for(t = 0;t < e2->knode2()->numEdges();t++)
+			for(t = 0;!sf && t < e2->node2()->numEdges();t++)
 			{
-				e3 = e2->knode2()->getEdge(t);
+				e3 = e2->node2()->getEdge(t);
 				if(e1 == e3 || e2 == e3 || del.find(e3) < 0)
 					continue;
 /*
 				printf("seg(%i,%i,%i) <%g,%g,%g><%g,%g,%g><%g,%g,%g>\n",e1->id(),e2->id(),e3->id(),
-					e1->knode1()->vector()[0],e1->knode1()->vector()[1],e1->knode1()->vector()[2],
-					e2->knode1()->vector()[0],e2->knode1()->vector()[1],e2->knode1()->vector()[2],
-					e3->knode1()->vector()[0],e3->knode1()->vector()[1],e3->knode1()->vector()[2]);
+					e1->node1()->vector()[0],e1->node1()->vector()[1],e1->node1()->vector()[2],
+					e2->node1()->vector()[0],e2->node1()->vector()[1],e2->node1()->vector()[2],
+					e3->node1()->vector()[0],e3->node1()->vector()[1],e3->node1()->vector()[2]);
 */
-				if(e2->knode2() != e3->knode1() || e3->knode2() != e1->knode1())
+				if(e2->node2() != e3->node1() || e3->node2() != e1->node1())
 					continue;
-				if(addSegment(e1,e2,e3))
+				s1 = getNodeIndex(e1->node1());
+				s2 = getNodeIndex(e2->node1());
+				s3 = getNodeIndex(e3->node1());
+				if(!hasSegment(s1,s2,s3))
 				{
-					del.deleteAt(del.find(e1));
-					del.deleteAt(del.find(e2));
-					del.deleteAt(del.find(e3));
-					e1 = del[c % del.length()];
+					if(addSegment(s1,s2,s3))
+					{
+						del.deleteAt(del.find(e1));
+						del.deleteAt(del.find(e2));
+						del.deleteAt(del.find(e3));
+//						e1 = del[c % del.length()];
+						sf = true;
+						break;
+					}
 				}
 			}
-			for(t = 0;t < e2->knode2()->numEdges();t++)
+			for(t = 0;!sf && t < e2->node2()->numEdges();t++)
 			{
-				e3 = e2->knode2()->getEdge(t)->swap();
+				e3 = e2->node2()->getEdge(t)->swap();
 				if(e1 == e3 || e2 == e3 || del.find(e3) < 0)
 					continue;
 /*
 				printf("seg(%i,%i,%i) <%g,%g,%g><%g,%g,%g><%g,%g,%g>\n",e1->id(),e2->id(),e3->id(),
-					e1->knode1()->vector()[0],e1->knode1()->vector()[1],e1->knode1()->vector()[2],
-					e2->knode1()->vector()[0],e2->knode1()->vector()[1],e2->knode1()->vector()[2],
-					e3->knode1()->vector()[0],e3->knode1()->vector()[1],e3->knode1()->vector()[2]);
+					e1->node1()->vector()[0],e1->node1()->vector()[1],e1->node1()->vector()[2],
+					e2->node1()->vector()[0],e2->node1()->vector()[1],e2->node1()->vector()[2],
+					e3->node1()->vector()[0],e3->node1()->vector()[1],e3->node1()->vector()[2]);
 */
-				if(e2->knode2() != e3->knode1() || e3->knode2() != e1->knode1())
+				if(e2->node2() != e3->node1() || e3->node2() != e1->node1())
 					continue;
-				if(addSegment(e1,e2,e3))
+				s1 = getNodeIndex(e1->node1());
+				s2 = getNodeIndex(e2->node1());
+				s3 = getNodeIndex(e3->node1());
+				if(!hasSegment(s1,s2,s3))
 				{
-					del.deleteAt(del.find(e1));
-					del.deleteAt(del.find(e2));
-					del.deleteAt(del.find(e3));
-					e1 = del[c % del.length()];
+					if(addSegment(s1,s2,s3))
+					{
+						del.deleteAt(del.find(e1));
+						del.deleteAt(del.find(e2));
+						del.deleteAt(del.find(e3));
+//						e1 = del[c % del.length()];
+						sf = true;
+						break;
+					}
 				}
 			}
 		}
@@ -393,23 +328,41 @@ int		cmesh::createFromGraph(graph *g)
 	return 1;
 }
 
+bool	cmesh::hasSegment(edge *e1,edge *e2,edge *e3)
+{
+	int		n,s1,s2,s3;
+
+       	s1 = s2 = s3 = -1;
+       	for(n = 0;n < vectorlist.length();n++)
+       	{
+       		if(vectorlist[n].position == e1->node1()->vector())
+       			s1 = n;
+       		if(vectorlist[n].position == e1->node2()->vector())
+       			s2 = n;
+       		if(vectorlist[n].position == e2->node2()->vector())
+       			s3 = n;
+       	}
+       	return hasSegment(s1,s2,s3);
+}
+
+
 int	cmesh::addSegment(edge *e1,edge *e2,edge *e3)
 {
 	int		n,s1,s2,s3;
 
 	if(e1 != e2 && e1 != e3 && e2 != e3
-		&& e1->knode2() == e2->knode1()
-		&& e2->knode2() == e3->knode1()
-		&& e3->knode2() == e1->knode1())
+		&& e1->node2() == e2->node1()
+		&& e2->node2() == e3->node1()
+		&& e3->node2() == e1->node1())
 	{
 		s1 = s2 = s3 = -1;
 		for(n = 0;n < vectorlist.length();n++)
 		{
-			if(vectorlist[n].position == e1->knode1()->vector())
+			if(vectorlist[n].position == e1->node1()->vector())
 				s1 = n;
-			if(vectorlist[n].position == e1->knode2()->vector())
+			if(vectorlist[n].position == e1->node2()->vector())
 				s2 = n;
-			if(vectorlist[n].position == e2->knode2()->vector())
+			if(vectorlist[n].position == e2->node2()->vector())
 				s3 = n;
 		}
 		addSegment(s1,s2,s3);
@@ -421,6 +374,62 @@ int	cmesh::addSegment(edge *e1,edge *e2,edge *e3)
 	return 0;
 }
 
+bool	cmesh::hasSegment(node *n1,node *n2,node *n3)
+{
+	int		n,s1,s2,s3;
+
+       	s1 = s2 = s3 = -1;
+       	for(n = 0;n < vectorlist.length();n++)
+       	{
+       		if(vectorlist[n].position == n1->vector())
+       			s1 = n;
+       		if(vectorlist[n].position == n2->vector())
+       			s2 = n;
+       		if(vectorlist[n].position == n3->vector())
+       			s3 = n;
+       	}
+       	return hasSegment(s1,s2,s3);
+}
+
+
+int	cmesh::addSegment(node *n1,node *n2,node *n3)
+{
+	int		n,s1,s2,s3;
+
+	if(n1 != n2 && n1 != n3 && n2 != n3)
+	{
+		s1 = s2 = s3 = -1;
+		for(n = 0;n < vectorlist.length();n++)
+		{
+			if(vectorlist[n].position == n1->vector())
+				s1 = n;
+			if(vectorlist[n].position == n2->vector())
+				s2 = n;
+			if(vectorlist[n].position == n3->vector())
+				s3 = n;
+		}
+		addSegment(s1,s2,s3);
+		printf("add seg (%i,%i,%i)\n",s1,s2,s3);
+
+		return 1;
+	}
+
+	return 0;
+}
+
+int	cmesh::getNodeIndex(node *n1)
+{
+	int		n;
+
+	for(n = 0;n < vectorlist.length();n++)
+	{
+		if(vectorlist[n].position == n1->vector())
+			return n;
+	}
+
+	return -1;
+}
+
 
 int		cmesh::isClose()
 {
@@ -448,10 +457,10 @@ int		cmesh::closeMesh()
 	g = createGraph();
 
 
-	printf("graph has %i points and %i edges\n",g->numKnodes(),g->numEdges());
+	printf("graph has %i points and %i edges\n",g->numNodes(),g->numEdges());
 
 /*
-	g->removeDoubleKnodes();
+	g->removeDoubleNodes();
 	g->removeDoubleEdges();
 */
 	if(g->isClose())
@@ -474,11 +483,11 @@ int		cmesh::closeMesh()
 			printf("is not close\n");
 		}
 
-		printf("graph has %i points and %i edges\n",g->numKnodes(),g->numEdges());
+		printf("graph has %i points and %i edges\n",g->numNodes(),g->numEdges());
 		return createFromGraph(g);
 	}
 
-	printf("graph has %i points and %i edges\n",g->numKnodes(),g->numEdges());
+	printf("graph has %i points and %i edges\n",g->numNodes(),g->numEdges());
 
 	createFromGraph(g);
 
@@ -495,29 +504,6 @@ int		cmesh::isConvex()
 	return 0;
 }
 
-int		cmesh::simplify(double grade)
-{
-	// check all 2 neighbourtriangles
-	// if there are some in a plane, simplify
-
-	return 0;
-}
-
-int		cmesh::subdivide()
-{
-	// take  every triangle and divide it into 2
-
-	return 0;
-}
-
-int		cmesh::subdivideClever(int base,double grade)
-{
-	// take every tiangle and divide it into base * base
-	// triangles
-	// take advantage of the normalvectors
-	
-	return 0;
-}
 
 int		cmesh::addMesh(cmesh&)
 {
@@ -573,7 +559,7 @@ int		cmesh::removeFromParent()
 void		cmesh::dumpNames(int tab,int)
 {
 	printTab(stdout,tab);
-	printf("cmesh: %s\n",name);
+	printf("cmesh: %s    ( %i nodes %i segments)\n",name,getNumNodes(),getNumSegments());
 }
 
 int		cmesh::exportPOVMesh(FILE *fp,int tab,int tabsize,int anim)
@@ -611,7 +597,7 @@ int		cmesh::exportPOVMesh(FILE *fp,int tab,int tabsize,int anim)
 	}
 	if(texptr) texptr->exportPOV(fp,tab + tabsize,tabsize,anim);
 
-	dim::exportPOV(fp,tab + tabsize,tabsize,anim);
+	transform::exportPOV(fp,tab + tabsize,tabsize,anim);
 
 	printTab(fp,tab);
 	fprintf(fp,"}\n\n");
@@ -654,155 +640,115 @@ int		cmesh::exportPOV(FILE *fp,int tab,int tabsize,int anim)
 	return 0;
 }
 
-int		cmesh::save(media *m)
+int		cmesh::save(media *m,int ver)
 {
 	int	i;
 
 	if(!m)
 		return -1;
 
-	setMedia(m);
-
-	writeChunk("CMSH");
-	writeNameChunk(name);
-
-	saveFlags(m);
-	anim::save(m);
-	dim::save(m);
-
-	writeInt(vectorlist.length());
-	for(i = 0;i < vectorlist.length();i++)
+	switch(ver)
 	{
-		writeVector(vectorlist[i].position);
-		writeVector(vectorlist[i].direction);
-	}
-	writeInt(trianglelist.length());
-	for(i = 0;i < trianglelist.length();i++)
-	{
-		writeInt(trianglelist[i].pointnum[0]);
-		writeInt(trianglelist[i].pointnum[1]);
-		writeInt(trianglelist[i].pointnum[2]);
-	}
+		case 0:
+		case -1:
+			setMedia(m);
 
-	saveTexture(m);
+			writeChunk("CMSH");
+			writeNameChunk(name);
 
-	writeChunkLen();
+			saveFlags(m);
+			anim::save(m);
+			transform::save(m,ver);
+
+			writeInt(vectorlist.length());
+			for(i = 0;i < vectorlist.length();i++)
+			{
+				writeVector(vectorlist[i].position);
+				writeVector(vectorlist[i].direction);
+			}
+			writeInt(trianglelist.length());
+			for(i = 0;i < trianglelist.length();i++)
+			{
+				writeInt(trianglelist[i].pointnum[0]);
+				writeInt(trianglelist[i].pointnum[1]);
+				writeInt(trianglelist[i].pointnum[2]);
+			}
+
+			saveTexture(m);
+
+			writeChunkLen();
+		break;
+		default:
+			return -2;
+	}
 
 	return 0;
 }
 
-int		cmesh::load(media *m,int l)
+int		cmesh::load(media *m,int l,int ver)
 {
 	int		i,n,s0,s1,s2;
 	int		pos;
 	Vector3		v,vn;
 
-	setMedia(m);
-
-	pos = m->tell();
-
-	loadFlags(m,l);
-
-	anim::load(m,l - (m->tell() - pos));
-	dim::load(m,l - (m->tell() - pos));
-
-	n = readInt();
-	for(i = 0;i < n;i++)
+	switch(ver)
 	{
-		v = readVector();
-		vn = readVector();
+		case 0:
+		case -1:
+			setMedia(m);
 
-		addKnode(v,vn);
-	}
+			pos = m->tell();
 
-	n = readInt();
-	for(i = 0;i < n;i++)
-	{
-		s0 = readInt(); 
-		s1 = readInt(); 
-		s2 = readInt(); 
-		addSegment(s0,s1,s2); 
-	}
+			loadFlags(m,l);
 
-	if(l - (m->tell() - pos) > 0)
-	{
-		loadTexture(m,l - (m->tell() - pos));
+			anim::load(m,l - (m->tell() - pos));
+			transform::load(m,l - (m->tell() - pos),ver);
+
+			n = readInt();
+			for(i = 0;i < n;i++)
+			{
+				v = readVector();
+				vn = readVector();
+
+				addNode(v,vn);
+			}
+
+			n = readInt();
+			for(i = 0;i < n;i++)
+			{
+				s0 = readInt();
+				s1 = readInt();
+				s2 = readInt();
+				addSegment(s0,s1,s2);
+			}
+
+			if(l - (m->tell() - pos) > 0)
+			{
+				loadTexture(m,l - (m->tell() - pos));
+			}
+ 		break;
+		default:
+			return -2;
 	}
 
 	return 0;
 }
 
 
-int		cmesh::draw(view *v,Matrix44 m,int anim)
-{
-	int		i;
-	seg		s;
-	Vector3		v0,v1,v2;
-
-	if(isFlag(HIDE)) return 0;
-
-	dimMatrix(m,anim);
-
-	if(this == v->getSelected()) v->setDrawSelected(1);
-
-	if(isFlag(DRAW_BOUNDINGBOX))
-		drawBB(v,m,anim);
-	else
-	{
-		for(i = 0;i < trianglelist.length();i++)
-		{
-			s = trianglelist[i];
-			v0 = vectorlist[s.pointnum[0]].position;
-			v1 = vectorlist[s.pointnum[1]].position;
-			v2 = vectorlist[s.pointnum[2]].position;
-			v->drawLine(v0,v1,m,anim);
-			v->drawLine(v1,v2,m,anim);
-			v->drawLine(v2,v0,m,anim);
-		}
-	}
-
-	if(this == v->getSelected()) v->setDrawSelected(0);
-
-	if(isFlag(DRAW_AXIS))
-		v->drawAxis(m,anim);
-
-	drawDragvectors(v,m,anim);
-
-
-	return 0;
-}
-
-int		cmesh::calculate(int)
-{
-	int		i;
-	double		x,y,z;
-
-	vmin = vmax = vectorlist[0].position;
-	for(i = 1;i < vectorlist.length();i++)
-	{
-		x = vectorlist[i].position[0];
-		y = vectorlist[i].position[1];
-		z = vectorlist[i].position[2];
-		vmin = Vector3(MIN(x,vmin[0]),MIN(y,vmin[1]),MIN(z,vmin[2]));
-		vmax = Vector3(MAX(x,vmax[0]),MAX(y,vmax[1]),MAX(z,vmax[2]));
-	}
-
-	return 0;
-}
 
 int		cmesh::addChild(box *b)
 {
 	vectorlist.empty();
 	trianglelist.empty();
 
-	addKnode(-0.5,-0.5,-0.5);
-	addKnode( 0.5,-0.5,-0.5);
-	addKnode(-0.5, 0.5,-0.5);
-	addKnode( 0.5, 0.5,-0.5);
-	addKnode(-0.5,-0.5, 0.5);
-	addKnode( 0.5,-0.5, 0.5);
-	addKnode(-0.5, 0.5, 0.5);
-	addKnode( 0.5, 0.5, 0.5);
+	addNode(-0.5,-0.5,-0.5);
+	addNode( 0.5,-0.5,-0.5);
+	addNode(-0.5, 0.5,-0.5);
+	addNode( 0.5, 0.5,-0.5);
+	addNode(-0.5,-0.5, 0.5);
+	addNode( 0.5,-0.5, 0.5);
+	addNode(-0.5, 0.5, 0.5);
+	addNode( 0.5, 0.5, 0.5);
 
 	addSegment(0,1,3);
 	addSegment(0,3,2);
@@ -830,19 +776,19 @@ int		cmesh::addChild(cone *c)
 	trianglelist.empty();
 
 
-	addKnode(0,0,0.5);
-	addKnode(0,0,-0.5);
+	addNode(0,0,0.5);
+	addNode(0,0,-0.5);
 	for(i = 0;i < s;i++)
 	{
 		a = (double)i / (double)s * 2 * PI;
 		v = Vector3(0.5 * sin(a),
 			0.5 * cos(a),
 			0.5);
-		addKnode(v);
+		addNode(v);
 		v = Vector3(c->capRadius() * sin(a),
 			c->capRadius() * cos(a),
 			-0.5);
-		addKnode(v);
+		addNode(v);
 	}
 
 	for(i = 0;i < s;i++)
@@ -870,19 +816,19 @@ int		cmesh::addChild(cylinder *c)
 	trianglelist.empty();
 
 
-	addKnode(0,0,0.5);
-	addKnode(0,0,-0.5);
+	addNode(0,0,0.5);
+	addNode(0,0,-0.5);
 	for(i = 0;i < s;i++)
 	{
 		a = (double)i / (double)s * 2 * PI;
 		v = Vector3(0.5 * sin(a),
 			0.5 * cos(a),
 			0.5);
-		addKnode(v);
+		addNode(v);
 		v = Vector3(0.5 * sin(a),
 			0.5 * cos(a),
 			-0.5);
-		addKnode(v);
+		addNode(v);
 	}
 
 	for(i = 0;i < s;i++)
@@ -910,8 +856,8 @@ int		cmesh::addChild(sphere *c)
 	trianglelist.empty();
 
 
-	addKnode(0,0.5,0);
-	addKnode(0,-0.5,0);
+	addNode(0,0.5,0);
+	addNode(0,-0.5,0);
 	for(t = 1;t < (s / 2);t++)
 	{
 		b = (double)t / (double)s * 2 * PI;
@@ -921,7 +867,7 @@ int		cmesh::addChild(sphere *c)
 			v = Vector3(0.5 * sin(a) * sin(b),
 				0.5 * cos(b),
 				0.5 * cos(a) * sin(b));
-			addKnode(v);
+			addNode(v);
 		}
 	}
 
@@ -964,7 +910,7 @@ int		cmesh::addChild(text *c)
 		return i;
 
 /*
-	printf("Got %i knodes and %i edges\n",vectorlist.length(),trianglelist.length() * 3);
+	printf("Got %i nodes and %i edges\n",vectorlist.length(),trianglelist.length() * 3);
 
 	for(i = 0;i < vectorlist.length();i++)
 		printf("%i. <%g,%g,%g>\n",i,vectorlist[i].position[0],vectorlist[i].position[1],vectorlist[i].position[2]);
@@ -1012,3 +958,176 @@ int		cmesh::addChild(blobobj *co)
 	return addChild((csgobj*)co);
 }
 
+int		cmesh::subdevide()
+{
+	int	error;
+		
+      	error = smesh::subdevide();
+		
+	if(gptr)
+	{
+		delete gptr;
+		gptr = 0;
+	}
+	
+      	if(error != 0)
+      		return error;
+			
+      	error = close();
+		
+#ifdef DEBUG
+      	printf("Mesh is %s after subdevision\n",(error == 0?"close":"not close"));
+#endif
+
+      	return error;
+}
+
+int		cmesh::subdevideClever(int base,double grade)
+{
+      	return smesh::subdevideClever(base,grade);
+}
+
+int		cmesh::simplify(double grade)
+{
+	graph		*g;
+        list<node*>	nodelist;
+        node		*np,*fn;
+        int		i;
+        Vector3		v1,v2,v3,vn;
+
+	g = createGraph();
+
+//	printf("graph has %i points and %i edges\n",g->numNodes(),g->numEdges());
+	printf("simplify\n");
+
+	if(!g->close())
+	{
+		printf("is not close => no simplyfication\n");
+		return -1;
+	}
+	
+	for(i = 0;i < g->numNodes();i++)
+	{
+		nodelist += g->getNode(i);
+	}
+	
+	fn = 0;
+	while(!nodelist.isEmpty())
+	{
+#ifdef DEBUG
+		printf("length nodelist %i\n",nodelist.length());
+#endif
+
+		np = nodelist[0];
+		if(!np)
+			break;
+		
+		if(np->numEdges() == 2)
+		{
+			printf("graph not close!!! double edge found.\n");
+			// replace these 2 edges by one new edge
+		}
+		else if(np->numEdges() == 3)
+		{
+			v1 = np->getEdge(0)->vector();
+			v2 = np->getEdge(1)->vector();
+			v3 = np->getEdge(2)->vector();
+			vn = v1 * v2;
+			if(vn.scalarprod(v3) < grade)
+			{
+				// delete the 3 edges
+				// delete the node
+				nodelist.deleteAt(nodelist.find(np));
+				delete np;
+			}
+		}
+		else
+		{
+			if(fn == np)
+				break;
+			if(!fn)
+				fn = np;
+			nodelist.moveToEnd(0);
+		}
+	}
+	
+	createFromGraph(g);
+	
+      	return 0;
+}
+
+int		cmesh::smooth()
+{
+        int		i,n;
+        node		*np;
+        Vector3		v,v1,v2,va,vn;
+        graph		*g;
+
+#ifdef DEBUG
+	printf("smooth: %i points and %i segments\n",vectorlist.length(),trianglelist.length());
+#endif
+
+	if(gptr)
+	{
+		delete gptr;
+		gptr = 0;
+	}
+	
+	g = createGraph();
+
+#ifdef DEBUG
+	printf("smooth: graph has %i points and %i edges\n",g->numNodes(),g->numEdges());
+#endif
+
+/*
+	if(!g->close())
+	{
+		printf("is not close => no smoothification\n");
+		return -1;
+	}
+*/
+	
+#ifdef DEBUG
+	printf("\n");
+#endif
+        for(i = 0;i < g->numNodes();i++)
+        {
+        	v = Vector3(0,0,0);
+        	np = g->getNode(i);
+        	va = np->vector();
+#ifdef DEBUG
+		printf("%2i.  (%g,%g,%g) + \n",i,va[0],va[1],va[2]);
+#endif
+        	for(n = 0;n < np->numEdges();n++)
+        	{
+        		v1 = np->getEdge(n)->node1()->vector();
+        		v2 = np->getEdge(n)->node2()->vector();
+        		if(v1 == va)
+        		{
+        			v += v2;
+#ifdef DEBUG
+				printf("     (%g,%g,%g) + \n",v2[0],v2[1],v2[2]);
+#endif
+        		}
+        		else
+        		{
+        			v += v1;
+#ifdef DEBUG
+				printf("     (%g,%g,%g) + \n",v1[0],v1[1],v1[2]);
+#endif
+        		}
+        	}
+        	v /= np->numEdges();
+        	vn = (v * 0.2) + (va * 0.8);
+        	np->setVector(vn);
+#ifdef DEBUG
+		printf("=    (%g,%g,%g)\n",vn[0],vn[1],vn[2]);
+#endif
+        }
+	
+	createFromGraph(g);
+	
+	return 0;
+}
+
+
