@@ -23,31 +23,26 @@
 /*
 ** dialogs.cpp
 */
+
 #include "dialogs.h"
 #include "dialogs.moc"
 
-#include <klocale.h>
-
-#define stloc(a)	((char*)locale->translate((a)))
-
+#include <kapp.h>
+#include <qvalidator.h>
 
 NameDialog::NameDialog(QWidget *parent=0,const char *name=0) :
 	QDialog(parent,name,TRUE)
 {
-	KLocale	*locale;
-
 	setCaption(name);
 
-	locale = new KLocale();
-
 	line = new QLineEdit(this);
 
-	ok = new QPushButton( this,stloc("Ok"));
-	ok->setText(stloc("OK"));
+	ok = new QPushButton( this,i18n("OK"));
+	ok->setText(i18n("OK"));
 	connect( ok, SIGNAL(clicked()), SLOT(accept()) );
 
-	cancel = new QPushButton( this,stloc("Cancel") );
-	cancel->setText(stloc("Cancel"));
+	cancel = new QPushButton( this,i18n("Cancel") );
+	cancel->setText(i18n("Cancel"));
 	connect( cancel, SIGNAL(clicked()), SLOT(reject()) );
 
 	setMaximumSize(150,65);
@@ -57,8 +52,6 @@ NameDialog::NameDialog(QWidget *parent=0,const char *name=0) :
 	line->setGeometry(5,5,140,25);
 	cancel->setGeometry(5,35,67,25);
 	ok->setGeometry(77,35,67,25);	
-
-	delete locale;
 }
 
 NameDialog::~NameDialog()
@@ -79,18 +72,69 @@ void	NameDialog::setName(QString s)
 }
 
 // ************************************************************************
+
+EarDialog::EarDialog(QWidget *parent=0,const char *name=0) :
+	QDialog(parent,name,TRUE)
+{
+	QIntValidator	*iv;
+	
+	setCaption(name);
+
+	line = new QLineEdit(this);
+	line->setValidator((iv = new QIntValidator(line)));
+ 	iv->setRange(0,255);
+
+	ok = new QPushButton( this,i18n("OK"));
+	ok->setText(i18n("OK"));
+	connect( ok, SIGNAL(clicked()), SLOT(accept()) );
+
+	cancel = new QPushButton( this,i18n("Cancel") );
+	cancel->setText(i18n("Cancel"));
+	connect( cancel, SIGNAL(clicked()), SLOT(reject()) );
+
+	setMaximumSize(150,65);
+	QDialog::resize(150,65);
+	setMinimumSize(150,65);
+
+	line->setGeometry(5,5,140,25);
+	cancel->setGeometry(5,35,67,25);
+	ok->setGeometry(77,35,67,25);	
+}
+
+EarDialog::~EarDialog()
+{
+}
+
+unsigned char	EarDialog::getChannel()
+{
+	unsigned char	c;
+	
+	c = (unsigned char)atoi(line->text());
+	
+	return c;
+}
+
+void	EarDialog::setChannel(unsigned char c)
+{
+	QString		qstr;
+	
+	qstr.setNum((unsigned short)c);
+	line->setText(qstr);
+}
+
+// ************************************************************************
+
+
 InfoDialog::InfoDialog(const char *name=0) :
 	QDialog(NULL,"Info",true)
 {
-
 	QSize qs;
 
-	setCaption("Info");
-
+	setCaption(i18n("Info"));
 
 	label = new QLabel(name,this);
 
-	ok = new QPushButton("Ok", this);
+	ok = new QPushButton(i18n("OK"), this);
 
 	connect( ok, SIGNAL(clicked()), SLOT(accept()) );
 
@@ -115,21 +159,17 @@ InfoDialog::~InfoDialog()
 CameraDialog::CameraDialog(QWidget *parent=0,const char *name=0) :
 	QDialog(parent,name,true)
 {
-	KLocale	*locale;
-
 	setCaption(name);
 
-	locale = new KLocale();
-
 	viewselect = new QComboBox(this);
 	viewselect->setInsertionPolicy(QComboBox::NoInsertion);
 
-	ok = new QPushButton( this,stloc("Ok"));
-	ok->setText(stloc("OK"));
+	ok = new QPushButton( this,i18n("OK"));
+	ok->setText(i18n("OK"));
 	connect( ok, SIGNAL(clicked()), SLOT(accept()) );
 
-	cancel = new QPushButton( this,stloc("Cancel") );
-	cancel->setText(stloc("Cancel"));
+	cancel = new QPushButton( this,i18n("Cancel") );
+	cancel->setText(i18n("Cancel"));
 	connect( cancel, SIGNAL(clicked()), SLOT(reject()) );
 
 	setMaximumSize(150,65);
@@ -139,8 +179,6 @@ CameraDialog::CameraDialog(QWidget *parent=0,const char *name=0) :
 	viewselect->setGeometry(5,5,140,25);
 	cancel->setGeometry(5,35,67,25);
 	ok->setGeometry(77,35,67,25);	
-
-	delete locale;
 }
 
 CameraDialog::~CameraDialog()
@@ -166,24 +204,21 @@ void	CameraDialog::setViewList(QStrList s)
 BlobDialog::BlobDialog(QWidget *parent=0,const char *name=0) :
 	QDialog(parent,name,TRUE)
 {
-	KLocale	*locale;
 	int	w,w2;
 
 	setCaption(name);
 
-	locale = new KLocale();
-
 	label = new QLabel(this);
-	label->setText("Thresholdvalue:");
+	label->setText(i18n("Thresholdvalue:"));
 
 	line = new QLineEdit(this);
 
-	ok = new QPushButton( this,stloc("Ok"));
-	ok->setText(stloc("OK"));
+	ok = new QPushButton( this,i18n("OK"));
+	ok->setText(i18n("OK"));
 	connect( ok, SIGNAL(clicked()), SLOT(accept()) );
 
-	cancel = new QPushButton( this,stloc("Cancel") );
-	cancel->setText(stloc("Cancel"));
+	cancel = new QPushButton( this,i18n("Cancel") );
+	cancel->setText(i18n("Cancel"));
 	connect( cancel, SIGNAL(clicked()), SLOT(reject()) );
 
 	w2 = label->sizeHint().width();
@@ -199,8 +234,6 @@ BlobDialog::BlobDialog(QWidget *parent=0,const char *name=0) :
 	w2 = w2 * 2 / 3;
 	cancel->setGeometry((w - 5) / 2 - w2,35,w2,25);
 	ok->setGeometry((w - 5) / 2 + 5,35,w2,25);	
-
-	delete locale;
 }
 
 BlobDialog::~BlobDialog()
@@ -228,109 +261,53 @@ void	BlobDialog::setThresholdValue(double t)
 // ******************************************************************************************
 
 
-DimDialog::DimDialog(QWidget *parent,const char *name) :
-	QDialog(parent,name,TRUE)
-{
-	KLocale	*locale;
-
-	setCaption(name);
-
-	dimw = new DimWidget(this,"");
-
-	locale = new KLocale();
-
-	ok = new QPushButton( this,stloc("Ok"));
-	ok->setText(stloc("OK"));
-	connect( ok, SIGNAL(clicked()), SLOT(accept()) );
-
-	cancel = new QPushButton( this,stloc("Cancel") );
-	cancel->setText(stloc("Cancel"));
-	connect( cancel, SIGNAL(clicked()), SLOT(reject()) );
-
-	setMaximumSize(300,140);
-	QDialog::resize(300,140);
-	setMinimumSize(300,140);
-
-	dimw->setGeometry(0,0,300,125);
-	cancel->setGeometry(67,110,80,25);
-	ok->setGeometry(153,110,80,25);	
-
-	delete locale;
-
-	dimw->layout();
-}
-
-DimDialog::~DimDialog()
-{
-	delete	ok;
-	delete	cancel;
-	delete	dimw;
-}
-
-void	DimDialog::save(Vector3 &s,Vector3 &r,Vector3 &t)
-{
-	dimw->save(s,r,t);
-}
-
-void	DimDialog::load(Vector3 s,Vector3 r,Vector3 t)
-{
-	dimw->load(s,r,t);
-}
-
-
-// ******************************************************************************************
-
-
 TextDialog::TextDialog(QWidget *parent,const char *name) :
 	QDialog(parent,name,TRUE)
 {
-	KLocale		*locale;
 	QString		path;
 	Preferences	prefs;
 
 	path = prefs.getPath(prefs.getFontPath());
 
-	locale = new KLocale();
-
 	setCaption(name);
 
 	lcontext = new QLabel(this);
-	lcontext->setText(stloc("Context:"));
+	lcontext->setText(i18n("Context:"));
 
 	wcontext = new QLineEdit(this);
 
 /*
 	lfontname = new QLabel(this);
-	lfontname->setText(stloc("Fontname:"));
+	lfontname->setText(i18n("Fontname:"));
 
 	wfontname = new QLineEdit(this);
 */
 
-	wfontname = new GetFileWidget(this,"Fontname:",path);
+	wfontname = new GetFileWidget(this,i18n("Fontname:"),path);
 
 	lthickness = new QLabel(this);
-	lthickness->setText(stloc("Thickness:"));
+	lthickness->setText(i18n("Thickness:"));
 
 	wthickness = new QLineEdit(this);
 
 	loffset = new QLabel(this);
-	loffset->setText(stloc("Offset:"));
+	loffset->setText(i18n("Offset:"));
 
 	wxoffset = new QLineEdit(this);
 	wyoffset = new QLineEdit(this);
 
 	
-	ok = new QPushButton( this,stloc("Ok"));
-	ok->setText(stloc("OK"));
+	ok = new QPushButton( this,i18n("OK"));
+	ok->setText(i18n("OK"));
 	connect( ok, SIGNAL(clicked()), SLOT(accept()) );
 
-	cancel = new QPushButton( this,stloc("Cancel") );
-	cancel->setText(stloc("Cancel"));
+	cancel = new QPushButton( this,i18n("Cancel") );
+	cancel->setText(i18n("Cancel"));
 	connect( cancel, SIGNAL(clicked()), SLOT(reject()) );
 
-	setMaximumSize(300,170);
-	QDialog::resize(300,170);
-	setMinimumSize(300,170);
+	setMaximumSize(400,170);
+	QDialog::resize(400,170);
+	setMinimumSize(400,170);
 
 	lcontext->setGeometry(5,5,140,25);
 	wcontext->setGeometry(145,5,150,25);
@@ -339,7 +316,7 @@ TextDialog::TextDialog(QWidget *parent,const char *name) :
 	lfontname->setGeometry(5,30,140,25);
 	wfontname->setGeometry(145,30,150,25);
 */
-	wfontname->setGeometry(5,30,290,25);
+	wfontname->setGeometry(5,30,390,25);
 
 	lthickness->setGeometry(5,60,140,25);
 	wthickness->setGeometry(145,60,150,25);
@@ -350,8 +327,6 @@ TextDialog::TextDialog(QWidget *parent,const char *name) :
 
 	cancel->setGeometry(67,130,80,25);
 	ok->setGeometry(153,130,80,25);	
-
-	delete locale;
 }
 
 TextDialog::~TextDialog()
@@ -403,68 +378,69 @@ void	TextDialog::load(QString context,QString fontname,double thickness,Vector2
 HeightFieldDialog::HeightFieldDialog(QWidget *parent,const char *name) :
 	QDialog(parent,name,TRUE)
 {
-	KLocale		*locale;
 	Preferences	prefs;
         QString		str;
 
-	locale = new KLocale();
-
 	setCaption(name);
 
 	lwaterlevel = new QLabel(this);
-	lwaterlevel->setText(stloc("Waterlevel:"));
+	lwaterlevel->setText(i18n("Waterlevel:"));
 
 	wwaterlevel = new QLineEdit(this);
 
 	lfilename = new QLabel(this);
-	lfilename->setText(stloc("Filename:"));
+	lfilename->setText(i18n("Filename:"));
 	
 	str = prefs.getPath(prefs.getHeightfieldPath());
 #ifdef DEBUG 	
  	printf("heightfield: %s\n",(const char*)str);
 #endif
- 	wfilename = new GetFileWidget(this,"",str);
+ 	wfilename = new GetFileWidget(this,(const char *)"",str);
 
+	csmooth = new QCheckBox(this);
+	csmooth->setText(i18n("Smooth"));
 	
-	ok = new QPushButton( this,stloc("Ok"));
-	ok->setText(stloc("OK"));
+	ok = new QPushButton( this,i18n("OK"));
+	ok->setText(i18n("OK"));
 	connect( ok, SIGNAL(clicked()), SLOT(accept()) );
 
-	cancel = new QPushButton( this,stloc("Cancel") );
-	cancel->setText(stloc("Cancel"));
+	cancel = new QPushButton( this,i18n("Cancel") );
+	cancel->setText(i18n("Cancel"));
 	connect( cancel, SIGNAL(clicked()), SLOT(reject()) );
 
-	setMaximumSize(370,170);
-	QDialog::resize(370,170);
-	setMinimumSize(370,170);
+	setMaximumSize(400,170);
+	QDialog::resize(400,170);
+	setMinimumSize(400,170);
 
 	lfilename->setGeometry(5,5,140,25);
- 	wfilename->setGeometry(140,5,230,25);
+ 	wfilename->setGeometry(140,5,250,25);
 
 	lwaterlevel->setGeometry(5,35,140,25);
 	wwaterlevel->setGeometry(145,35,150,25);
 
+	csmooth->setGeometry(5,75,190,25);
+	
 	cancel->setGeometry(102,130,80,25);
 	ok->setGeometry(188,130,80,25);	
-
-	delete locale;
 }
 
 HeightFieldDialog::~HeightFieldDialog()
 {
 }
 
-void	HeightFieldDialog::save(double *waterlevel,QString &filename)
+void	HeightFieldDialog::save(double &waterlevel,QString &filename,bool &smooth)
 {
 	QString		str;
 
 	str = wwaterlevel->text();
-	*waterlevel = str.toDouble();
+	waterlevel = str.toDouble();
 
 	filename = wfilename->getFilename();
+	
+	smooth = csmooth->isChecked();
 }
 
-void	HeightFieldDialog::load(double waterlevel,QString filename)
+void	HeightFieldDialog::load(double waterlevel,QString filename,bool smooth)
 {
 	QString		str;
 
@@ -472,6 +448,8 @@ void	HeightFieldDialog::load(double waterlevel,QString filename)
 	wwaterlevel->setText(str);
 
 	wfilename->setFilename(filename);
+	
+	csmooth->setChecked(smooth);
 }
 
 
@@ -485,33 +463,29 @@ void	HeightFieldDialog::load(double waterlevel,QString filename)
 LightPointDialog::LightPointDialog(QWidget *parent,const char *name) :
 	QDialog(parent,name,TRUE)
 {
-	KLocale	*locale;
-
-	locale = new KLocale();
-
 	setCaption(name);
 
 	lfadepower = new QLabel(this);
-	lfadepower->setText(stloc("Fadepower:"));
+	lfadepower->setText(i18n("Fadepower:"));
 
 	wfadepower = new QLineEdit(this);
 
 	lfadedistance = new QLabel(this);
-	lfadedistance->setText(stloc("Fadedistance:"));
+	lfadedistance->setText(i18n("Fadedistance:"));
 
 	wfadedistance = new QLineEdit(this);
 
 	lcolor = new QLabel(this);
-	lcolor->setText(stloc("Color:"));
+	lcolor->setText(i18n("Color:"));
 
 	wcolor = new KColorButton(this);
 
-	ok = new QPushButton( this,stloc("Ok"));
-	ok->setText(stloc("OK"));
+	ok = new QPushButton( this,i18n("OK"));
+	ok->setText(i18n("OK"));
 	connect( ok, SIGNAL(clicked()), SLOT(accept()) );
 
-	cancel = new QPushButton( this,stloc("Cancel") );
-	cancel->setText(stloc("Cancel"));
+	cancel = new QPushButton( this,i18n("Cancel") );
+	cancel->setText(i18n("Cancel"));
 	connect( cancel, SIGNAL(clicked()), SLOT(reject()) );
 
 	setMaximumSize(300,170);
@@ -529,8 +503,6 @@ LightPointDialog::LightPointDialog(QWidget *parent,const char *name) :
 
 	cancel->setGeometry(67,130,80,25);
 	ok->setGeometry(153,130,80,25);	
-
-	delete locale;
 }
 
 LightPointDialog::~LightPointDialog()
@@ -572,6 +544,160 @@ void	LightPointDialog::load(double fadepower,double fadedistance,Vector3 color)
 
 
 
+
+
+// ******************************************************************************************
+
+
+
+
+SpotLightDialog::SpotLightDialog(QWidget *parent,const char *name) :
+	QDialog(parent,name,TRUE)
+{
+	setCaption(name);
+
+	lfadepower = new QLabel(this);
+	lfadepower->setText(i18n("Fadepower:"));
+
+	wfadepower = new QLineEdit(this);
+
+	lfadedistance = new QLabel(this);
+	lfadedistance->setText(i18n("Fadedistance:"));
+
+	wfadedistance = new QLineEdit(this);
+
+	lcolor = new QLabel(this);
+	lcolor->setText(i18n("Color:"));
+
+	wcolor = new KColorButton(this);
+
+	lminteraction = new QLabel(this);
+        lminteraction->setText(i18n("Mediainteraction:"));
+
+	cminteraction = new QCheckBox(this);
+
+	lmattenuation = new QLabel(this);
+        lmattenuation->setText(i18n("Mediaattenuation:"));
+
+	cmattenuation = new QCheckBox(this);
+
+	lradius = new QLabel(this);
+	lradius->setText(i18n("Radius:"));
+
+	wradius = new QLineEdit(this);
+
+	lfalloff = new QLabel(this);
+	lfalloff->setText(i18n("Falloff:"));
+
+	wfalloff = new QLineEdit(this);
+
+	ltightness = new QLabel(this);
+	ltightness->setText(i18n("Tightness:"));
+
+	wtightness = new QLineEdit(this);
+
+	ok = new QPushButton( this,i18n("OK"));
+	ok->setText(i18n("OK"));
+	connect( ok, SIGNAL(clicked()), SLOT(accept()) );
+
+	cancel = new QPushButton( this,i18n("Cancel") );
+	cancel->setText(i18n("Cancel"));
+	connect( cancel, SIGNAL(clicked()), SLOT(reject()) );
+
+	setMaximumSize(300,290);
+	QDialog::resize(300,290);
+	setMinimumSize(300,290);
+
+	lfadedistance->setGeometry(5,5,140,25);
+	wfadedistance->setGeometry(145,5,150,25);
+
+	lfadepower->setGeometry(5,35,140,25);
+	wfadepower->setGeometry(145,35,150,25);
+
+	lcolor->setGeometry(5,70,140,25);
+	wcolor->setGeometry(145,70,150,25);
+
+	lminteraction->setGeometry(5,105,140,25);
+	cminteraction->setGeometry(145,105,140,25);
+	lmattenuation->setGeometry(5,135,140,25);
+	cmattenuation->setGeometry(145,135,140,25);
+
+	lradius->setGeometry(5,165,140,25);
+	wradius->setGeometry(145,165,150,25);
+
+	lfalloff->setGeometry(5,195,140,25);
+	wfalloff->setGeometry(145,195,150,25);
+
+	ltightness->setGeometry(5,225,140,25);
+	wtightness->setGeometry(145,225,150,25);
+
+	cancel->setGeometry(67,260,80,25);
+	ok->setGeometry(153,260,80,25);	
+}
+
+SpotLightDialog::~SpotLightDialog()
+{
+}
+
+void	SpotLightDialog::save(double &radius,double &falloff,double &tightness,double &fadepower,double &fadedistance,Vector3 &color,bool &mi,bool &ma)
+{
+	QString		str;
+	QColor		qc;
+
+	str = wradius->text();
+	radius = str.toDouble();
+
+	str = wfalloff->text();
+	falloff = str.toDouble();
+
+	str = wfalloff->text();
+	falloff = str.toDouble();
+
+	str = wtightness->text();
+	tightness = str.toDouble();
+
+	str = wfadedistance->text();
+	fadedistance = str.toDouble();
+
+	qc = wcolor->color();
+	color[0] = ((double)qc.red()) / 255.0;
+	color[1] = ((double)qc.green()) / 255.0;
+	color[2] = ((double)qc.blue()) / 255.0;
+	
+	mi = cminteraction->isChecked();
+	ma = cmattenuation->isChecked();
+}
+
+void	SpotLightDialog::load(double radius,double falloff,double tightness,double fadepower,double fadedistance,Vector3 color,bool mi,bool ma)
+{
+	QString		str;
+	QColor		qc;
+
+	str.setNum(radius);
+	wradius->setText(str);
+
+	str.setNum(falloff);
+	wfalloff->setText(str);
+
+	str.setNum(tightness);
+	wtightness->setText(str);
+
+	str.setNum(fadepower);
+	wfadepower->setText(str);
+
+	str.setNum(fadedistance);
+	wfadedistance->setText(str);
+
+	qc.setRgb(color[0] * 255,color[1] * 255,color[2] * 255);
+	wcolor->setColor(qc);
+	
+	cminteraction->setChecked(mi);
+	cmattenuation->setChecked(ma);
+}
+
+
+
+
 // ******************************************************************************************
 
 
@@ -580,15 +706,12 @@ void	LightPointDialog::load(double fadepower,double fadedistance,Vector3 color)
 AnimDialog::AnimDialog(QWidget *parent,const char *name,anim *a) :
 	QDialog(parent,name,TRUE)
 {
-	KLocale		*locale;
 	Preferences	prefs;
 //	int		n;
 	const char	**nl = (const char**)a->getScriptList();
 
 	_anim = a;
 
-	locale = new KLocale();
-
 	setCaption(name);
 
 	fname = new QComboBox(TRUE,this);
@@ -597,18 +720,18 @@ AnimDialog::AnimDialog(QWidget *parent,const char *name,anim *a) :
 	fname->setEditText("");			// provides nonconfusion
 	connect(fname,SIGNAL(activated(const char*)),this,SLOT(selectScript(const char*)));
 	
-	ok = new QPushButton(this,stloc("Ok"));
-	ok->setText(stloc("OK"));
+	ok = new QPushButton(this,i18n("OK"));
+	ok->setText(i18n("OK"));
 	connect(ok,SIGNAL(clicked()),SLOT(saveText()));
 	connect(ok,SIGNAL(clicked()),SLOT(accept()));
 
-	add = new QPushButton( this,stloc("Add") );
-	add->setText(stloc("Add"));
+	add = new QPushButton( this,i18n("Add") );
+	add->setText(i18n("Add"));
 	connect(add,SIGNAL(clicked()),SLOT(addScript()));
 
-	del = new QPushButton( this,stloc("Delete") );
+	del = new QPushButton( this,i18n("Delete") );
 //	del->setEnabled(FALSE);
-	del->setText(stloc("Delete"));
+	del->setText(i18n("Delete"));
 	connect(del,SIGNAL(clicked()),SLOT(delScript()));
 
 	mle = new QMultiLineEdit(this);
@@ -618,8 +741,6 @@ AnimDialog::AnimDialog(QWidget *parent,const char *name,anim *a) :
 	setMinimumSize(300,250);
 
 	
-	delete locale;
-
 /*
 	for(n = 0;nl[n] != 0;n++)
 		free(nl[n]);
@@ -639,6 +760,7 @@ void	AnimDialog::paintEvent(QPaintEvent *)
 	layout();
 	
 	prefs.setAnimEditorSize(width(),height());
+	prefs.save();
 }
 
 void	AnimDialog::layout()
@@ -646,7 +768,7 @@ void	AnimDialog::layout()
 	int	x,y,w,h,w2;
 	
 	x = y = 5;
-	w2 = 60;
+	w2 = MAX(add->sizeHint().width(),MAX(del->sizeHint().width(),ok->sizeHint().width()));
 	w = width() - w2 * 3 - 25;
 	h = 25;
 	fname->setGeometry(x,y,w,h);
@@ -802,14 +924,91 @@ void	AnimDialog::saveText()
 
 
 
+
+
+
+
+// ******************************************************************************************
+
+
+
+
+ScriptDialog::ScriptDialog(QWidget *parent,const char *name,script *s) :
+	QDialog(parent,name,TRUE)
+{
+	Preferences	prefs;
+
+	scriptobj = s;
+
+	setCaption(name);
+
+	ok = new QPushButton(this,i18n("OK"));
+	ok->setText(i18n("OK"));
+	connect(ok,SIGNAL(clicked()),SLOT(saveText()));
+	connect(ok,SIGNAL(clicked()),SLOT(accept()));
+
+	cancel = new QPushButton( this,i18n("Cancel") );
+	cancel->setText(i18n("Cancel"));
+	connect(cancel,SIGNAL(clicked()),SLOT(reject()));
+
+	mle = new QMultiLineEdit(this);
+	if(scriptobj)
+		mle->setText(scriptobj->getText());
+
+	setMaximumSize(1024,960);
+//	QDialog::resize(prefs.getAnimEditorWidth(),prefs.getAnimEditorHeight());
+	setMinimumSize(300,250);
+
+	layout();
+}
+
+ScriptDialog::~ScriptDialog()
+{
+}
+
+void	ScriptDialog::paintEvent(QPaintEvent *)
+{
+	Preferences	prefs;
+	
+	layout();
+	
+//	prefs.setAnimEditorSize(width(),height());
+//	prefs.save();
+}
+
+void	ScriptDialog::layout()
+{
+	int	x,y,w,h;
+		
+	x = 5;
+	y = 5;
+	w = width() - 10;
+	h = height() - 15 - 25;
+	mle->setGeometry(x,y,w,h);
+	
+	w = MAX(cancel->sizeHint().width(),ok->sizeHint().width());
+	h = 25;
+	x = width() - 10 - w * 2;
+	y = height() - 5 - h;
+	ok->setGeometry(x,y,w,h);
+	x += w + 5;
+	cancel->setGeometry(x,y,w,h);
+}
+
+void	ScriptDialog::saveText()
+{
+	if(scriptobj)
+		scriptobj->setText(mle->text());
+}
+
+
+
 // ******************************************************************************************
 
 
 WorldDialog::WorldDialog(QWidget *parent,const char *name) :
 	QDialog(parent,name,TRUE)
 {
-	KLocale	*locale;
-
 	setCaption(name);
 
 	lex = new QLineEdit(this,"");
@@ -822,21 +1021,19 @@ WorldDialog::WorldDialog(QWidget *parent,const char *name) :
 	ambient_light = Vector3(0,0,0);
 
 	allabel = new QLabel(this,"");
-	allabel->setText("Ambient light:");
+	allabel->setText(i18n("Ambient light:"));
 
 	bgc_label = new QLabel(this,"");
-	bgc_label->setText("Backgroundcolor:");
+	bgc_label->setText(i18n("Backgroundcolor:"));
 
 	backg_color = new KColorButton(this);
 
-	locale = new KLocale();
-
-	ok = new QPushButton( this,stloc("Ok"));
-	ok->setText(stloc("OK"));
+	ok = new QPushButton( this,i18n("OK"));
+	ok->setText(i18n("OK"));
 	connect( ok, SIGNAL(clicked()), SLOT(accept()) );
 
-	cancel = new QPushButton( this,stloc("Cancel") );
-	cancel->setText(stloc("Cancel"));
+	cancel = new QPushButton( this,i18n("Cancel") );
+	cancel->setText(i18n("Cancel"));
 	connect( cancel, SIGNAL(clicked()), SLOT(reject()) );
 
 	setMaximumSize(300,140);
@@ -853,8 +1050,6 @@ WorldDialog::WorldDialog(QWidget *parent,const char *name) :
 
 	cancel->setGeometry(67,110,80,25);
 	ok->setGeometry(153,110,80,25);	
-
-	delete locale;
 }
 
 WorldDialog::~WorldDialog()
