@@ -26,9 +26,13 @@
 #include "preferencedialog.h"
 #include "preferencedialog.moc"
 
-#include <klocale.h>
+#include <kapp.h>
+#include <misc.h>
+
+#if (USE_PVM == 1)
+#include <pvm3.h>
+#endif
 
-#define stloc(a)	((char*)locale->translate((a)))
 
 
 
@@ -36,180 +40,161 @@ PreferencesDialog::PreferencesDialog(QWidget *pr) :
 	QTabDialog(pr,"Preferences",true)
 {
 	QColor qc;
+	int i;	
 
+	setCaption(i18n("Preferences"));
 	qc.setRgb(255,0,0);
 
-//	pathpage = new PathWidget(this);
 	pathpage = new QWidget(this);
 	editorpage = new QWidget(this);
 	renderpage = new QWidget(this);
 	screenpage = new QWidget(this);
+	miscpage = new QWidget(this);
+	clusterpage=new QWidget(this);
+
+	addTab(pathpage,i18n("Paths"));
+	addTab(editorpage,i18n("Editor"));
+	addTab(renderpage,i18n("Render"));
+	addTab(screenpage,i18n("Screen"));
+ 	addTab(miscpage,i18n("Misc"));
+#ifdef USE_PVM
+ 	addTab(clusterpage,i18n("PVM"));
+#endif
 
 	render1 = new QGroupBox(renderpage);
-	render1->setTitle("Render options");
+	render1->setTitle(i18n("Render options") );
 	render1->setGeometry(5,5,390,130);
 
 	render2 = new QGroupBox(renderpage);
-	render2->setTitle("Render options (Editor Preview)");
+	render2->setTitle(i18n("Render options (Editor Preview)") );
 	render2->setGeometry(5,140,390,130);
 
 	render3 = new QGroupBox(renderpage);
-	render3->setTitle("Render options (Texture Preview)");
+	render3->setTitle(i18n("Render options (Texture Preview)") );
 	render3->setGeometry(5,275,390,110);
 
-	addTab(pathpage,"Paths");
-	addTab(editorpage,"Editor");
-	addTab(renderpage,"Render");
-	addTab(screenpage,"Screen");
-
-	DataPathWidget = new GetFileWidget(pathpage,"",prefs.getDataPath(),2);
+	DataPathWidget = new GetFileWidget(pathpage,i18n("3D-Data"),prefs.getDataPath(),2);
 	DataPathWidget->setGeometry(85,5,280,25);
-	DataPathLabel = new QLabel(pathpage);
-	DataPathLabel->setText("3D-Data");
-	DataPathLabel->setGeometry(5,5,80,25);
 
-	TexturePathWidget = new GetFileWidget(pathpage,"",prefs.getTexturePath(),2);
+	TexturePathWidget = new GetFileWidget(pathpage,i18n("Textures"),prefs.getTexturePath(),2);
 	TexturePathWidget->setGeometry(85,35,280,25);
-	TexturePathLabel = new QLabel(pathpage);
-	TexturePathLabel->setText("Textures");	
-	TexturePathLabel->setGeometry(5,35,80,25);
 
-	imageMapPathWidget = new GetFileWidget(pathpage,"",prefs.getImageMapPath(),2);
+	imageMapPathWidget = new GetFileWidget(pathpage,i18n("Imagemaps"),prefs.getImageMapPath(),2);
 	imageMapPathWidget->setGeometry(85,65,280,25);
-	imageMapPathLabel = new QLabel(pathpage);
-	imageMapPathLabel->setText("Imagemaps");
-	imageMapPathLabel->setGeometry(5,65,80,25);
 
-	BumpMapPathWidget = new GetFileWidget(pathpage,"",prefs.getBumpMapPath(),2);
+	BumpMapPathWidget = new GetFileWidget(pathpage,i18n("Bumpmaps"),prefs.getBumpMapPath(),2);
 	BumpMapPathWidget->setGeometry(85,95,280,25);
-	BumpMapPathLabel = new QLabel(pathpage);
-	BumpMapPathLabel->setText("Bumpmaps");
-	BumpMapPathLabel->setGeometry(5,95,80,25);
 
-	fontPathWidget = new GetFileWidget(pathpage,"",prefs.getFontPath(),2);
+	fontPathWidget = new GetFileWidget(pathpage,i18n("Truetypefonts"),prefs.getFontPath(),2);
 	fontPathWidget->setGeometry(85,125,280,25);
-	fontPathLabel = new QLabel(pathpage);
-	fontPathLabel->setText("Truetypefonts");
-	fontPathLabel->setGeometry(5,125,80,25);
 
-	heightfieldPathWidget = new GetFileWidget(pathpage,"",prefs.getHeightfieldPath(),2);
+	heightfieldPathWidget = new GetFileWidget(pathpage,i18n("Heightfields"),prefs.getHeightfieldPath(),2);
 	heightfieldPathWidget->setGeometry(85,125,280,25);
-	heightfieldPathLabel = new QLabel(pathpage);
-	heightfieldPathLabel->setText("Heightfields");
-	heightfieldPathLabel->setGeometry(5,125,80,25);
 
-	ImagePathWidget = new GetFileWidget(pathpage,"",prefs.getImagePath(),2);
+	ImagePathWidget = new GetFileWidget(pathpage,i18n("Images"),prefs.getImagePath(),2);
 	ImagePathWidget->setGeometry(85,155,280,25);
-	ImagePathLabel = new QLabel(pathpage);
-	ImagePathLabel->setText("Images");
-	ImagePathLabel->setGeometry(5,155,80,25);
 
-	AnimationPathWidget = new GetFileWidget(pathpage,"",prefs.getAnimationPath(),2);
+	AnimationPathWidget = new GetFileWidget(pathpage,i18n("Animations"),prefs.getAnimationPath(),2);
 	AnimationPathWidget->setGeometry(85,185,280,25);
-	AnimationPathLabel = new QLabel(pathpage);
-	AnimationPathLabel->setText("Animations");
-	AnimationPathLabel->setGeometry(5,185,80,25);
 	
-	TempPathWidget = new GetFileWidget(pathpage,"",prefs.getTempPath(),2);
+	TempPathWidget = new GetFileWidget(pathpage,i18n("Temporary"),prefs.getTempPath(),2);
 	TempPathWidget->setGeometry(85,215,280,25);
-	TempPathLabel = new QLabel(pathpage);
-	TempPathLabel->setText("Tempory");
-	TempPathLabel->setGeometry(5,215,80,25);
 
+	
 
 
 	GridColorLabel = new QLabel(editorpage);
-	GridColorLabel->setText("Grid:");
+	GridColorLabel->setText(i18n("Grid:") );
 	GridColorLabel->setGeometry(5,5,90,25);
 	GridColor = new KColorButton (qc,editorpage);
 	GridColor->setColor(prefs.getGridColor());
 	GridColor->setGeometry(100,5,35,25);
 
 	XAxisColorLabel = new QLabel(editorpage);
-	XAxisColorLabel->setText("X-Axis:");
+	XAxisColorLabel->setText(i18n("X-Axes:") );
 	XAxisColorLabel->setGeometry(5,35,90,25);
 	XAxisColor = new KColorButton (qc,editorpage);
 	XAxisColor->setColor(prefs.getXAxisColor());	
 	XAxisColor->setGeometry(100,35,35,25);
 
 	YAxisColorLabel = new QLabel(editorpage);
-	YAxisColorLabel->setText("Y-Axis:");
+	YAxisColorLabel->setText(i18n("Y-Axes:") );
 	YAxisColorLabel->setGeometry(5,65,90,25);
 	YAxisColor = new KColorButton (qc,editorpage);
 	YAxisColor->setColor(prefs.getYAxisColor());;
 	YAxisColor->setGeometry(100,65,35,25);
 
 	ZAxisColorLabel = new QLabel(editorpage);
-	ZAxisColorLabel->setText("Z-Axis:");
+	ZAxisColorLabel->setText(i18n("Z-Axes:") );
 	ZAxisColorLabel->setGeometry(5,95,90,25);
 	ZAxisColor = new KColorButton (qc,editorpage);
 	ZAxisColor->setColor(prefs.getZAxisColor());;
 	ZAxisColor->setGeometry(100,95,35,25);
 
 	BackgroundColorLabel = new QLabel(editorpage);
-	BackgroundColorLabel->setText("Background:");
+	BackgroundColorLabel->setText(i18n("Background:") );
 	BackgroundColorLabel->setGeometry(5,125,90,25);
 	BackgroundColor = new KColorButton (qc,editorpage);	
 	BackgroundColor->setColor(prefs.getBackgroundColor());;
 	BackgroundColor->setGeometry(100,125,35,25);
 
 	LineColorLabel = new QLabel(editorpage);
-	LineColorLabel->setText("Lines:");
+	LineColorLabel->setText(i18n("Lines:") );
 	LineColorLabel->setGeometry(5,155,90,25);
 	LineColor = new KColorButton (qc,editorpage);	
 	LineColor->setColor(prefs.getLinesColor());;
 	LineColor->setGeometry(100,155,35,25);
 
 	SelectedLineColorLabel = new QLabel(editorpage);
-	SelectedLineColorLabel->setText("selected Lines:");
+	SelectedLineColorLabel->setText(i18n("selected Lines:") );
 	SelectedLineColorLabel->setGeometry(5,185,90,25);
 	SelectedLineColor = new KColorButton (qc,editorpage);	
 	SelectedLineColor->setColor(prefs.getSelectedLinesColor());;
 	SelectedLineColor->setGeometry(100,185,35,25);
 
 	BlobColorLabel = new QLabel(editorpage);
-	BlobColorLabel->setText("Blob:");
+	BlobColorLabel->setText(i18n("Blob:") );
 	BlobColorLabel->setGeometry(5,215,90,25);
 	BlobColor = new KColorButton (qc,editorpage);	
 	BlobColor->setColor(prefs.getBlobColor());
 	BlobColor->setGeometry(100,215,35,25);
 
 	BoxColorLabel = new QLabel(editorpage);
-	BoxColorLabel->setText("Boxes:");
+	BoxColorLabel->setText(i18n("Boxes:") );
 	BoxColorLabel->setGeometry(5,245,90,25);
 	BoxColor = new KColorButton (qc,editorpage);
 	BoxColor->setColor(prefs.getBoxColor());
 	BoxColor->setGeometry(100,245,35,25);
 
 	LetterXColorLabel = new QLabel(editorpage);
-	LetterXColorLabel->setText("X-Label:");
+	LetterXColorLabel->setText(i18n("X-Label:") );
 	LetterXColorLabel->setGeometry(150,35,65,25);
 	LetterXColor = new KColorButton (qc,editorpage);
 	LetterXColor->setColor(prefs.getLetterXColor());
 	LetterXColor->setGeometry(220,35,35,25);
 
 	LetterYColorLabel = new QLabel(editorpage);
-	LetterYColorLabel->setText("Y-Label:");
+	LetterYColorLabel->setText(i18n("Y-Label:") );
 	LetterYColorLabel->setGeometry(150,65,65,25);
 	LetterYColor = new KColorButton (qc,editorpage);
 	LetterYColor->setColor(prefs.getLetterYColor());
 	LetterYColor->setGeometry(220,65,35,25);
 
 	LetterZColorLabel = new QLabel(editorpage);
-	LetterZColorLabel->setText("Z-Label:");
+	LetterZColorLabel->setText(i18n("Z-Label:") );
 	LetterZColorLabel->setGeometry(150,95,65,25);
 	LetterZColor = new KColorButton (qc,editorpage);
 	LetterZColor->setColor(prefs.getLetterZColor());
 	LetterZColor->setGeometry(220,95,35,25);
 
 	DrawAxisCheckBox = new QCheckBox(editorpage);
-	DrawAxisCheckBox->setText("Draw Axes");
+	DrawAxisCheckBox->setText(i18n("Draw Axis") );
 	DrawAxisCheckBox->setGeometry(5,275,80,25);
 	DrawAxisCheckBox->setChecked(prefs.getDrawAxes() );
 
 //Renderer fr richtige Berechnung
 
-	RenderProgram = new GetFileWidget(render1,"Program",prefs.getRenderProgram(),0);
+	RenderProgram = new GetFileWidget(render1,i18n("Program") ,prefs.getRenderProgram(),0);
 	RenderProgram->setGeometry(5,15,390,25);
 
 	RenderParams = new QComboBox(true, render1);
@@ -221,7 +206,7 @@ PreferencesDialog::PreferencesDialog(QWidget *pr) :
 
 // Renderer fr Preview im Editor
 
-	RenderPreviewProgram = new GetFileWidget(render2,"Program",prefs.getRenderPreviewProgram(),0);
+	RenderPreviewProgram = new GetFileWidget(render2,i18n("Program") ,prefs.getRenderPreviewProgram(),0);
 	RenderPreviewProgram->setGeometry(5,10,390,25);
 
 	RenderPreviewParams = new QComboBox(true, render2);
@@ -233,7 +218,7 @@ PreferencesDialog::PreferencesDialog(QWidget *pr) :
 
 // Renderer fr Texturansicht
 
-	RenderSmallPreviewProgram = new GetFileWidget(render3,"Program:",prefs.getRenderSmallPreviewProgram(),0);
+	RenderSmallPreviewProgram = new GetFileWidget(render3,i18n("Program:") ,prefs.getRenderSmallPreviewProgram(),0);
 	RenderSmallPreviewProgram->setGeometry(5,10,390,25);
 
 	RenderSmallPreviewParams = new QComboBox(true, render3);
@@ -247,7 +232,7 @@ PreferencesDialog::PreferencesDialog(QWidget *pr) :
 	QString		str;
 
 	lscreensize = new QLabel(screenpage);
-	lscreensize->setText("Screensize (mm):");
+	lscreensize->setText(i18n("Screensize (mm):"));
 
 	screenwidth = new QLineEdit(screenpage);
 	str.setNum(prefs.getScreenWidth());
@@ -257,7 +242,7 @@ PreferencesDialog::PreferencesDialog(QWidget *pr) :
 	screenheight->setText(str);
 
 	lresolution = new QLabel(screenpage);
-	lresolution->setText("Resolution (pixel):");
+	lresolution->setText(i18n("Screenresolution (pixel):"));
 
 	reswidth = new QLineEdit(screenpage);
 	str.setNum(prefs.getResolutionWidth());
@@ -268,77 +253,155 @@ PreferencesDialog::PreferencesDialog(QWidget *pr) :
 
 
 	lgridspace_xy = new QLabel(screenpage);
-	lgridspace_xy->setText("Gridspace XY:");
+	lgridspace_xy->setText(i18n("Gridspace XY:"));
 
 	gridspace_xy = new QLineEdit(screenpage);
 	str.setNum(prefs.getGridSpaceXY());
 	gridspace_xy->setText(str);
 
 	lgridspace_xz = new QLabel(screenpage);
-	lgridspace_xz->setText("Gridspace XZ:");
+	lgridspace_xz->setText(i18n("Gridspace XZ:"));
 
 	gridspace_xz = new QLineEdit(screenpage);
 	str.setNum(prefs.getGridSpaceXZ());
 	gridspace_xz->setText(str);
 
 	lgridspace_zy = new QLabel(screenpage);
-	lgridspace_zy->setText("Gridspace ZY:");
+	lgridspace_zy->setText(i18n("Gridspace ZY:"));
 
 	gridspace_zy = new QLineEdit(screenpage);
 	str.setNum(prefs.getGridSpaceZY());
 	gridspace_zy->setText(str);
 
 	lgridlines_xy = new QLabel(screenpage);
-	lgridlines_xy->setText("Gridlines XY:");
+	lgridlines_xy->setText(i18n("Gridlines XY:"));
 
 	gridlines_xy = new QLineEdit(screenpage);
 	str.setNum(prefs.getGridLinesXY());
 	gridlines_xy->setText(str);
 
-	grid_xy = new QCheckBox("in perspective View",screenpage);
+	grid_xy = new QCheckBox(i18n("in perspective View"),screenpage);
 	grid_xy->setChecked(prefs.getGridXY());
 
 	lgridlines_xz = new QLabel(screenpage);
-	lgridlines_xz->setText("Gridlines XZ:");
+	lgridlines_xz->setText(i18n("Gridlines XZ:"));
 
 	gridlines_xz = new QLineEdit(screenpage);
 	str.setNum(prefs.getGridLinesXZ());
 	gridlines_xz->setText(str);
 
-	grid_xz = new QCheckBox("in perspective View",screenpage);
+	grid_xz = new QCheckBox(i18n("in perspective View"),screenpage);
 	grid_xz->setChecked(prefs.getGridXZ());
 
 	lgridlines_zy = new QLabel(screenpage);
-	lgridlines_zy->setText("Gridlines ZY:");
+	lgridlines_zy->setText(i18n("Gridlines ZY:"));
 
 	gridlines_zy = new QLineEdit(screenpage);
 	str.setNum(prefs.getGridLinesZY());
 	gridlines_zy->setText(str);
 
-	grid_zy = new QCheckBox("in perspective View",screenpage);
+	grid_zy = new QCheckBox(i18n("in perspective View"),screenpage);
 	grid_zy->setChecked(prefs.getGridZY());
 
 
-	resize(400,400);
+	autosavepicture = new QCheckBox(i18n("automatic save rendered picture"),miscpage);
+	autosavepicture->setChecked(prefs.getAutoSavePicture());
+
+	translatepov = new QCheckBox(i18n("translate POVRay-terms too"),miscpage);
+	translatepov->setChecked(prefs.getTranslatePOV());
+
+	// PVM-Einstellungen
+
+	clusterListView=new QListBox(clusterpage);
+	QToolTip::add(clusterListView,i18n("All Clusterlements wich should be used") );
+
+	for(i = 0;i < prefs.numClusterHosts();i++)
+	{
+		clusterListView->insertItem(prefs.recentClusterHost(i));
+	}
+
+	connect(clusterListView,SIGNAL(selected(const char*)),this,SLOT(clusterSelect(const char*) ) );
+
+	newClusterLine=new QLineEdit(clusterpage);
+	QToolTip::add(newClusterLine,i18n("Hostname/IP-Adress of a new Clusterelement") );	
+
+	speedClusterLine=new QLineEdit(clusterpage);
+	QToolTip::add(speedClusterLine,i18n("Relative speed of the host") );	
+
+	addCluster=new QPushButton(i18n("Add / Change"),clusterpage);
+	QToolTip::add(addCluster,i18n("add a Clusterelement") );
+	connect(addCluster,SIGNAL(clicked()),this,SLOT(addClusterPressed()));
+
+	deleteCluster=new QPushButton(i18n("Delete"),clusterpage);
+	QToolTip::add(deleteCluster,i18n("Delete Clusterelement") );
+	connect(deleteCluster,SIGNAL(clicked()),this,SLOT(deleteClusterPressed()));
+
+	showClusterStat=new QCheckBox(i18n("Show statistics"),clusterpage);
+	showClusterStat->setChecked(prefs.getShowClusterStatistics());
+	QToolTip::add(showClusterStat,i18n("Show statistics during renderprocess") );
+
+	resize(450,400);
 
 
 	if(!hasCancelButton())
-		setCancelButton("Cancel");
+		setCancelButton(i18n("Cancel") );
 	if(!hasOkButton())
-		setOkButton("Ok");
+		setOkButton(i18n("OK") );
 
 	connect(this,SIGNAL(applyButtonPressed()),this,SLOT(Apply()));
 	connect(this,SIGNAL(cancelButtonPressed()),this,SLOT(Cancel()));
+
+
 	
 	paintEvent(0);
 }
 
+void PreferencesDialog::clusterSelect(const char*c)
+{
+char *a,*b;
+
+	a=strtok( (char*)c,"=");
+	b=strtok(NULL,"=");
+	//if (b==NULL) speedClusterLine->setText("50");
+	//speedClusterLine->setText(b);
+	newClusterLine->setText(a);
+
+}
+
+void PreferencesDialog::deleteClusterPressed()
+{
+	if (clusterListView->currentItem()==-1) return;
+	clusterListView->removeItem(clusterListView->currentItem());
+}
+
+void PreferencesDialog::addClusterPressed()
+{
+char *txt=(char *)malloc(1024);
+int i;
+	if (!(strcmp(newClusterLine->text(),""))) return;
+
+	i=0;
+	for (i=0;i<(int)clusterListView->count();i++) {
+		if (!(strcmp(strtok( (char*)clusterListView->text(i),"="),newClusterLine->text()))) {
+			clusterListView->removeItem(i);
+		}
+	}
+	//if (!(strcmp(speedClusterLine->text(),""))) sprintf(txt,"%s=50",newClusterLine->text() );
+	//else sprintf(txt,"%s=%s",newClusterLine->text(),speedClusterLine->text() );
+	sprintf(txt,"%s",newClusterLine->text());	
+	clusterListView->insertItem(txt);
+	free(txt);
+}
+
 PreferencesDialog::~PreferencesDialog()
 {
+	delete autosavepicture;
 }
 
 void PreferencesDialog::Apply()
 {
+int i;
+
 	prefs.setTexturePath(TexturePathWidget->getFilename());
 	prefs.setTempPath(TempPathWidget->getFilename());
 	prefs.setImagePath(ImagePathWidget->getFilename());
@@ -389,7 +452,6 @@ void PreferencesDialog::Apply()
 	prefs.setResolution(w,h);
 
 	double	d;
-	int	i;
 	
 	str = gridlines_xy->text();
 	i = str.toInt();
@@ -416,6 +478,18 @@ void PreferencesDialog::Apply()
 	prefs.setGridSpaceZY(d);
 
 	
+	prefs.setAutoSavePicture(autosavepicture->isChecked());
+	prefs.setTranslatePOV(translatepov->isChecked());
+
+	prefs.setShowClusterStatistics(showClusterStat->isChecked() );
+
+	prefs.deleteClusterHostList();
+	for (i=0;i<(int)clusterListView->count();i++)
+	{
+		prefs.setClusterHost( clusterListView->text(i));
+	}
+
+		
 	prefs.save();
 
 	emit update();
@@ -433,6 +507,9 @@ void	PreferencesDialog::paintEvent(QPaintEvent *pe)
 	renderpage->setGeometry(10,10,width() - 20,height() - 20);
 	editorpage->setGeometry(10,10,width() - 20,height() - 20);
 	screenpage->setGeometry(10,10,width() - 20,height() - 20);
+	miscpage->setGeometry(10,10,width() - 20,height() - 20);
+	clusterpage->setGeometry(10,10,width() - 20,height() - 20);
+
 	
 	x = 5;
 	y = 5;
@@ -485,51 +562,49 @@ void	PreferencesDialog::paintEvent(QPaintEvent *pe)
 
 	x = 10;
 	y = 5;
-	w = 80;
-	w2 = pathpage->width() - 25 - w;
+	w = MAX(DataPathWidget->getLabelWidthHint(),0);
+	w = MAX(TexturePathWidget->getLabelWidthHint(),w);
+	w = MAX(imageMapPathWidget->getLabelWidthHint(),w);
+	w = MAX(BumpMapPathWidget->getLabelWidthHint(),w);
+	w = MAX(fontPathWidget->getLabelWidthHint(),w);
+	w = MAX(heightfieldPathWidget->getLabelWidthHint(),w);
+	w = MAX(ImagePathWidget->getLabelWidthHint(),w);
+	w = MAX(AnimationPathWidget->getLabelWidthHint(),w);
+	w = MAX(TempPathWidget->getLabelWidthHint(),w);
+	w2 = pathpage->width() - 20;
 	h = 25;
-	x2 = x + w + 5;
-
-	DataPathLabel->setGeometry(x,y,w,h);
-	DataPathWidget->setGeometry(x2,y,w2,h);
-
+	DataPathWidget->setLabelWidth(w);
+	DataPathWidget->setGeometry(x,y,w2,h);
 	y += h + 5;
-	TexturePathLabel->setGeometry(x,y,w,h);
-	TexturePathWidget->setGeometry(x2,y,w2,h);
-
+	TexturePathWidget->setLabelWidth(w);
+	TexturePathWidget->setGeometry(x,y,w2,h);
 	y += h + 5;
-	imageMapPathLabel->setGeometry(x,y,w,h);
-	imageMapPathWidget->setGeometry(x2,y,w2,h);
-
+	imageMapPathWidget->setLabelWidth(w);
+	imageMapPathWidget->setGeometry(x,y,w2,h);
 	y += h + 5;
-	BumpMapPathLabel->setGeometry(x,y,w,h);
-	BumpMapPathWidget->setGeometry(x2,y,w2,h);
-
+	BumpMapPathWidget->setLabelWidth(w);
+	BumpMapPathWidget->setGeometry(x,y,w2,h);
 	y += h + 5;
-	fontPathLabel->setGeometry(x,y,w,h);
-	fontPathWidget->setGeometry(x2,y,w2,h);
-
+	fontPathWidget->setLabelWidth(w);
+	fontPathWidget->setGeometry(x,y,w2,h);
 	y += h + 5;
-	heightfieldPathLabel->setGeometry(x,y,w,h);
-	heightfieldPathWidget->setGeometry(x2,y,w2,h);
-
+	heightfieldPathWidget->setLabelWidth(w);
+	heightfieldPathWidget->setGeometry(x,y,w2,h);
 	y += h + 5;
-	ImagePathLabel->setGeometry(x,y,w,h);
-	ImagePathWidget->setGeometry(x2,y,w2,h);
-
+	ImagePathWidget->setLabelWidth(w);
+	ImagePathWidget->setGeometry(x,y,w2,h);
 	y += h + 5;
-	AnimationPathLabel->setGeometry(x,y,w,h);
-	AnimationPathWidget->setGeometry(x2,y,w2,h);
-	
+	AnimationPathWidget->setLabelWidth(w);
+	AnimationPathWidget->setGeometry(x,y,w2,h);
 	y += h + 5;
-	TempPathLabel->setGeometry(x,y,w,h);
-	TempPathWidget->setGeometry(x2,y,w2,h);
+	TempPathWidget->setLabelWidth(w);
+	TempPathWidget->setGeometry(x,y,w2,h);
 
 /***********************************************************************/
 	
 	x = 10;
 	y = 5;
-	w = 90;
+	w = 120;
 	w2 = 35;
 	h = 25;
 	x2 = x + w + 5;
@@ -662,6 +737,50 @@ void	PreferencesDialog::paintEvent(QPaintEvent *pe)
 	gridspace_zy->setGeometry(x,y,w2,h);
 
 /***********************************************************************/
+   	
+ 	x = 10;
+	y = 5;
+	w = miscpage->width() - 20;
+	h = 25;
+	autosavepicture->setGeometry(x,y,w,h);
+	
+ 	x = 10;
+	y += h + 5;
+	w = miscpage->width() - 20;
+	h = 25;
+	translatepov->setGeometry(x,y,w,h);
+	
+/***********************************************************************/
+
+	w=clusterpage->width();
+	h=clusterpage->height();
+	x=0;
+	y=0;
+	x2=0;
+
+	clusterListView->setGeometry(10,5,w-20,h/2);
+	y+=h/2+10;
+
+	h=newClusterLine->sizeHint().height();
+
+	newClusterLine->setGeometry(10,y,width()/10*8-5,h);
+	speedClusterLine->setGeometry(10+width()/10*8,y,width()/10*2,h);
+	
+	y+=h+5;
+
+	x=addCluster->sizeHint().width();
+	h=addCluster->sizeHint().height();
+	x2=deleteCluster->sizeHint().width();
+
+	addCluster->setGeometry(10,y,x,h);
+	deleteCluster->setGeometry(15+x,y,x2,h);
+
+	y+=h+5;
+
+	w=showClusterStat->sizeHint().width();
+	h=showClusterStat->sizeHint().height();
+	
+	showClusterStat->setGeometry(10,y,w,h);
 
 	if(pe) QTabDialog::paintEvent(pe);
 }
