@@ -26,16 +26,27 @@
 #include "viewwidget.h"
 #include "viewwidget.moc"
 
+
+#include <kapp.h>
+
 #include <math.h>
 #include <misc.h>
 
-ViewWidget::ViewWidget(QWidget *p,const char *n,world *wptr) :
-	QWidget(p,n), view(wptr,n)
+ViewWidget::ViewWidget(QWidget *p,const char *n,world *wptr,bool autodel) :
+	QFrame(p,n), view(wptr,n,autodel)
 {
-	qpm = new QPixmap(width(),height());
-	qp = new QPainter(qpm);
+#if (USE_RTTI == 0)
+	bv = 0;
+#endif
+#if (USE_PVM == 1)
+	pvm_on=false;
+#endif
 
-	configure();
+	setFrameStyle( Box | Sunken );
+        setLineWidth(1);
+
+	qpm = new QPixmap(width() - 4,height() - 4);
+	qp = new QPainter(qpm);
 
 	peye = Vector3(0,0,200);
 	vview = Vector3(0,0,-1);
@@ -53,13 +64,20 @@ ViewWidget::ViewWidget(QWidget *p,const char *n,world *wptr) :
 	axemode = 7;
 
 	ppmfp = 0;
+	tcounter = -1;
+
+	configure();
 }
 
 ViewWidget::~ViewWidget()
 {
+	emit destroyed();
+	
 	qp->end();
 	
 	delete qp;
+	
+	delete qpm;
 }
 
 int	ViewWidget::drawLine(Vector3 a,Vector3 b,Matrix44 m,int anim)
@@ -344,9 +362,9 @@ int		ViewWidget::drawCone(Vector3 m,Vector3 a,Vector3 b,Vector3 c,double d,Matri
 	drawLine(v[2],v[6],ma,anim);
 	drawLine(v[3],v[7],ma,anim);
 
-	drawCircle(c,a,b,ma,anim);
+	drawCircle(m + c,a,b,ma,anim);
 	drawCircle(m,a * (1 + d) / 2,b * (1 + d) / 2,ma,anim);
-	drawCircle(-c,a * d,b * d,ma,anim);
+	drawCircle(m - c,a * d,b * d,ma,anim);
 
 	return 0;
 }
@@ -524,7 +542,19 @@ void		ViewWidget::draw(int anim)
 
 	m.unify();
 
-	qp->fillRect(1,1,width() - 2,height() - 2,backcolor);
+	if(getSubObject())
+	{
+		vm = -((transform*)subobject)->getVTranslate();
+		m.translateVector(vm);
+	}
+	else
+	{
+		vm = Vector3(0,0,0);
+	}
+			
+	qp->fillRect(0,0,width(),height(),backcolor);
+
+/*
 	qp->setPen(QColor(0,0,0));
 	qp->drawLine(0,0,width(),0);
 	qp->drawLine(0,0,0,height());
@@ -532,6 +562,7 @@ void		ViewWidget::draw(int anim)
 	qp->setPen(QColor(255,255,255));
 	qp->drawLine(width() - 1,0,width() - 1,height() - 1);
 	qp->drawLine(0,height() - 1,width() - 1,height() - 1);
+*/
 
 	qp->setPen(gridcolor);
 
@@ -540,97 +571,102 @@ void		ViewWidget::draw(int anim)
 		case PERSPECTIVE:
 			for(t = -gridlines_xz;grid_xz && t <= gridlines_xz;t++)
 			{
-				a = (vx * (double)t + vz * -gridlines_xz) * gridspace_xz;
-				b = (vx * (double)t + vz * gridlines_xz) * gridspace_xz;
+				a = vm + (vx * (double)t + vz * -gridlines_xz) * gridspace_xz;
+				b = vm + (vx * (double)t + vz * gridlines_xz) * gridspace_xz;
 				drawLine(a,b,m,anim,gridcolor);
-				a = (vz * (double)t + vx * -gridlines_xz) * gridspace_xz;
-				b = (vz * (double)t + vx * gridlines_xz) * gridspace_xz;
+				a = vm + (vz * (double)t + vx * -gridlines_xz) * gridspace_xz;
+				b = vm + (vz * (double)t + vx * gridlines_xz) * gridspace_xz;
 				drawLine(a,b,m,anim,gridcolor);
 			}
 			for(t = -gridlines_xy;grid_xy && t <= gridlines_xy;t++)
 			{
-				a = (vx * (double)t + vy * -gridlines_xy) * gridspace_xy;
-				b = (vx * (double)t + vy * gridlines_xy) * gridspace_xy;
+				a = vm + (vx * (double)t + vy * -gridlines_xy) * gridspace_xy;
+				b = vm + (vx * (double)t + vy * gridlines_xy) * gridspace_xy;
 				drawLine(a,b,m,anim,gridcolor);
-				a = (vy * (double)t + vx * -gridlines_xy) * gridspace_xy;
-				b = (vy * (double)t + vx * gridlines_xy) * gridspace_xy;
+				a = vm + (vy * (double)t + vx * -gridlines_xy) * gridspace_xy;
+				b = vm + (vy * (double)t + vx * gridlines_xy) * gridspace_xy;
 				drawLine(a,b,m,anim,gridcolor);
 			}
 			for(t = -gridlines_zy;grid_zy && t <= gridlines_zy;t++)
 			{
-				a = (vz * (double)t + vy * -gridlines_zy) * gridspace_zy;
-				b = (vz * (double)t + vy * gridlines_zy) * gridspace_zy;
+				a = vm + (vz * (double)t + vy * -gridlines_zy) * gridspace_zy;
+				b = vm + (vz * (double)t + vy * gridlines_zy) * gridspace_zy;
 				drawLine(a,b,m,anim,gridcolor);
-				a = (vy * (double)t + vz * -gridlines_zy) * gridspace_zy;
-				b = (vy * (double)t + vz * gridlines_zy) * gridspace_zy;
+				a = vm + (vy * (double)t + vz * -gridlines_zy) * gridspace_zy;
+				b = vm + (vy * (double)t + vz * gridlines_zy) * gridspace_zy;
 				drawLine(a,b,m,anim,gridcolor);
 			}
 
-			drawLine(vm,vx * 10,m,anim,xaxecolor);//QColor(200,0,0)
+			drawLine(vm,vm + vx * 10,m,anim,xaxecolor);//QColor(200,0,0)
 
-			drawLine(vx * 10 + vx + vz,vx * 10 - vz,m,anim,letterxcolor);//QColor(100,0,0));
-			drawLine(vx * 10 + vx - vz,vx * 10 + vz,m,anim,letterxcolor);
+			drawLine(vm + vx * 10 + vx + vz,vm + vx * 10 - vz,m,anim,letterxcolor);//QColor(100,0,0));
+			drawLine(vm + vx * 10 + vx - vz,vm + vx * 10 + vz,m,anim,letterxcolor);
 
-			drawLine(vm,vy * 10,m,anim,yaxecolor);//QColor(0,200,0)
+			drawLine(vm,vm + vy * 10,m,anim,yaxecolor);//QColor(0,200,0)
 
-			drawLine(vy * 10 + vy + vx,vy * 10 + vy * 0.5,m,anim,letterycolor);//QColor(0,100,0));
-			drawLine(vy * 10 +      vx,vy * 10 + vy * 0.5,m,anim,letterycolor);
-			drawLine(vy * 10 + vy * 0.5,vy * 10 + vy * 0.5 - vx,m,anim,letterycolor);
+			drawLine(vm + vy * 10 + vy + vx,vm + vy * 10 + vy * 0.5,m,anim,letterycolor);//QColor(0,100,0));
+			drawLine(vm + vy * 10 +      vx,vm + vy * 10 + vy * 0.5,m,anim,letterycolor);
+			drawLine(vm + vy * 10 + vy * 0.5,vm + vy * 10 + vy * 0.5 - vx,m,anim,letterycolor);
 
-			drawLine(vm,vz * 10,m,anim,zaxecolor);//QColor(100,0,200)
+			drawLine(vm,vm + vz * 10,m,anim,zaxecolor);//QColor(100,0,200)
 
-			drawLine(vz * 10 + vz + vx,vz * 10 + vx,m,anim,letterzcolor);//QColor(50,0,100));
-			drawLine(vz * 10 + vz - vx,vz * 10 - vx,m,anim,letterzcolor);
-			drawLine(vz * 10 + vz - vx,vz * 10 + vx,m,anim,letterzcolor);
+			drawLine(vm + vz * 10 + vz + vx,vm + vz * 10 + vx,m,anim,letterzcolor);//QColor(50,0,100));
+			drawLine(vm + vz * 10 + vz - vx,vm + vz * 10 - vx,m,anim,letterzcolor);
+			drawLine(vm + vz * 10 + vz - vx,vm + vz * 10 + vx,m,anim,letterzcolor);
 		break;
 		case PARALLEL_XY:
 			for(t = -gridlines_xy;t <= gridlines_xy;t++)
 			{
-				a = (vx * (double)t + vy * -10) * gridspace_xy;
-				b = (vx * (double)t + vy * 10) * gridspace_xy;
+				a = vm + (vx * (double)t + vy * -10) * gridspace_xy;
+				b = vm + (vx * (double)t + vy * 10) * gridspace_xy;
 				drawLine(a,b,m,anim,gridcolor);
-				a = (vy * (double)t + vx * -10) * gridspace_xy;
-				b = (vy * (double)t + vx * 10) * gridspace_xy;
+				a = vm + (vy * (double)t + vx * -10) * gridspace_xy;
+				b = vm + (vy * (double)t + vx * 10) * gridspace_xy;
 				drawLine(a,b,m,anim,gridcolor);
 			}
 
-			drawLine(vm,vx * 10,m,anim,xaxecolor);
-			drawLine(vm,vy * 10,m,anim,yaxecolor);
+			drawLine(vm,vm + vx * 10,m,anim,xaxecolor);
+			drawLine(vm,vm + vy * 10,m,anim,yaxecolor);
 		break;
 		case PARALLEL_ZY:
 			for(t = -gridlines_zy;t <= gridlines_zy;t++)
 			{
-				a = (vz * (double)t + vy * -10) * gridspace_zy;
-				b = (vz * (double)t + vy * 10) * gridspace_zy;
+				a = vm + (vz * (double)t + vy * -10) * gridspace_zy;
+				b = vm + (vz * (double)t + vy * 10) * gridspace_zy;
 				drawLine(a,b,m,anim,gridcolor);
-				a = (vy * (double)t + vz * -10) * gridspace_zy;
-				b = (vy * (double)t + vz * 10) * gridspace_zy;
+				a = vm + (vy * (double)t + vz * -10) * gridspace_zy;
+				b = vm + (vy * (double)t + vz * 10) * gridspace_zy;
 				drawLine(a,b,m,anim,gridcolor);
 			}
 
-			drawLine(vm,vz * 10,m,anim,zaxecolor);
-			drawLine(vm,vy * 10,m,anim,yaxecolor);
+			drawLine(vm,vm + vz * 10,m,anim,zaxecolor);
+			drawLine(vm,vm + vy * 10,m,anim,yaxecolor);
 		break;
 		case PARALLEL_XZ:
 			for(t = -gridlines_xz;t <= gridlines_xz;t++)
 			{
-				a = (vx * (double)t + vz * -10) * gridspace_xz;
-				b = (vx * (double)t + vz * 10) * gridspace_xz;
+				a = vm + (vx * (double)t + vz * -10) * gridspace_xz;
+				b = vm + (vx * (double)t + vz * 10) * gridspace_xz;
 				drawLine(a,b,m,anim,gridcolor);
-				a = (vz * (double)t + vx * -10) * gridspace_xz;
-				b = (vz * (double)t + vx * 10) * gridspace_xz;
+				a = vm + (vz * (double)t + vx * -10) * gridspace_xz;
+				b = vm + (vz * (double)t + vx * 10) * gridspace_xz;
 				drawLine(a,b,m,anim,gridcolor);
 			}
 
-			drawLine(vm,vz * 10,m,anim,zaxecolor);
-			drawLine(vm,vx * 10,m,anim,xaxecolor);
+			drawLine(vm,vm + vz * 10,m,anim,zaxecolor);
+			drawLine(vm,vm + vx * 10,m,anim,xaxecolor);
 		break;
 	}
 
 	qp->setPen(drawcolor);
 	
 	if(worldptr)
-		worldptr->draw(this,m,anim);
+	{
+		if(getSubObject())
+			subobject->draw(this,m,anim);
+		else
+			worldptr->draw(this,m,anim);
+	}
 	else
 		printf("no world!\n");
 
@@ -640,18 +676,76 @@ void		ViewWidget::draw(int anim)
 void		ViewWidget::setWorld(world *ww)
 {
 	view	*v;
-	
-	v = ww->getView(0);
-	
-	if(v)
-	{
-		ww->removeView(v);
-		take(v);
+        int	i,error;
 
-		delete v;
-	}
-	view::setWorld(ww);
+	// is this world new for me?
+	if(ww == getWorld())
+		return; // no, nothing to do
+
+	// lets search a view without widget
+	for(i = 0;i < ww->getNumViews();i++)
+	{			
+		v = ww->getView(i);
 	
+		// lets have a look at v
+		// if v is a widget
+#if (USE_RTTI == 1)
+		if(dynamic_cast<ViewWidget*>(v) != 0)
+#else
+		if(v->baseview() == 0)
+#endif
+			continue; // take the next one
+			
+		// Do not remove myself !!!
+		// (we should not reach here, if v is widget)
+		if((view*)this != v)
+		{
+			ww->removeView(v);
+			take(v);
+			
+			delete v;
+			break;
+		}
+	}
+	
+	while((error = view::setWorld(ww)) != 0)
+	{
+		switch(error)
+		{
+#ifdef DEBUG		
+			case -1:
+				printf("Unknown error in %s at line %i\n",__FILE__,__LINE__);
+			break;
+#endif
+			case -2:
+#ifdef DEBUG		
+				printf("view has no name in %s at line %i\n",__FILE__,__LINE__);
+#endif
+			break;
+#ifdef DEBUG		
+			case -3:
+				printf("view exist in %s at line %i\n",__FILE__,__LINE__);
+			break;
+#endif
+			case -4:
+#ifdef DEBUG		
+				printf("view has existing name in %s at line %i\n",__FILE__,__LINE__);
+#endif
+				{
+					char	buffer[256];
+				
+				        strncpy(buffer,getName(),254);
+				        strcat(buffer,"1");
+				        setName(buffer);
+				
+				        emit changedViewName();
+				 }
+			break;
+		}
+	}	
+	
+//	resize();
+		
 	draw();
 }
 
@@ -742,11 +836,12 @@ void		ViewWidget::mouseMoveEvent(QMouseEvent *me)
 	{
 		if(selectedptr->changeDragvector(this,selectedptr->getMatrix(),xo,yo,me->x(),me->y()) == 0)
 		{
-			draw();
-
 			xo = me->x();
 			yo = me->y();
 
+			emit changedModel();
+			emit redraw();
+			
 			return;
 		}
 	}
@@ -782,26 +877,29 @@ void		ViewWidget::mouseMoveEvent(QMouseEvent *me)
 				v(1) = exp((double)dx / -100);
 			if(axemode & 4)
 				v(2) = exp((double)dx / -100);
-			((dim*)selectedptr)->scale(v);
+			((transform*)selectedptr)->scale(v);
 			draw();
+			emit changedModel();
 		break;
 		case 5: // Object translate
 			if(!selectedptr || selectedptr->getType() == NUM_WORLD) return;
 			if(me->state() == LeftButton)
-				((dim*)selectedptr)->translate(Vector3((double)dx / 10.0,(double)dy / 10.0,0));
+				((transform*)selectedptr)->translate(Vector3((double)dx / 10.0,(double)dy / 10.0,0));
 			else
-				((dim*)selectedptr)->translate(Vector3(0,(double)dy / 10.0,(double)dx / 10.0));
+				((transform*)selectedptr)->translate(Vector3(0,(double)dy / 10.0,(double)dx / 10.0));
 			draw();
+			emit changedModel();
 		break;
 		case 6: // Object rotate
 			if(!selectedptr || selectedptr->getType() == NUM_WORLD) return;
 			if(axemode & 1)
-				((dim*)selectedptr)->rotate(Vector3((double)dx / 1800 * PI,0,0));
+				((transform*)selectedptr)->rotate(Vector3((double)dx / 1800 * PI,0,0));
 			if(axemode & 2)
-				((dim*)selectedptr)->rotate(Vector3(0,0,(double)dx / 1800 * PI));
+				((transform*)selectedptr)->rotate(Vector3(0,0,(double)dx / 1800 * PI));
 			if(axemode & 4)
-				((dim*)selectedptr)->rotate(Vector3(0,(double)dx / 1800 * PI,0));
+				((transform*)selectedptr)->rotate(Vector3(0,(double)dx / 1800 * PI,0));
 			draw();
+			emit changedModel();
 		break;
 	}
 	draw();
@@ -817,18 +915,27 @@ void		ViewWidget::mousePressEvent(QMouseEvent *me)
 }
 
 
-void		ViewWidget::paintEvent(QPaintEvent*)
+void		ViewWidget::resizeEvent(QResizeEvent *re)
 {
-	setScreenSize(width(),height());
-	createScreen(width(),height());
-
+	setScreenSize(width() - 4,height() - 4);
+	createScreen(width() - 4,height() - 4);
+	
 	qp->end();
-	qpm->resize(width(),height());
+	qpm->resize(width() - 4,height() - 4);
 	qp->begin(qpm);
 
 	draw();
 	
-	emit resizedView(width(),height());
+	emit resizedView(width() - 4,height() - 4);
+	
+	QFrame::resizeEvent(re);
+}
+
+void		ViewWidget::paintEvent(QPaintEvent *pe)
+{
+	flush();
+	
+	QFrame::paintEvent(pe);
 }
 
 
@@ -837,40 +944,60 @@ void		ViewWidget::flush()
 	QPainter	paint(this);
 
 	qp->end();
-	paint.drawPixmap(0,0,*qpm);
+	paint.drawPixmap(2,2,*qpm);
 	qp->begin(qpm);
 }
 
+
+#if (USE_PVM == 0)
+// NON-PVM-Version !!
+
+
 void		ViewWidget::startRendering(int quality)
 {
 	char		str[16];
-	QString		qstr,tmp;
-	Preferences	pref;
+	QString		qstr;
+	int		viewnum;
+	Preferences	prefs;
 
-	killTimers();
-
-	qstr = "Start rendering with ";
-	qstr += pref.getRenderProgram();
-	emit report(qstr);
+	// now wait a quarter second
+	if(tcounter != -1 && tcounter < 5)
+		return;
+		
+	printf("Start Rendering\n");
 	
-	tmp = pref.getTempPath();
+	// stop all the timer of this widget
+	killTimers();
+	proc.kill();
+	if(ppmfp)
+		fclose(ppmfp);
+	ppmfp = 0;
+	flush();
 
-	qstr = tmp + "/tmp.pov";
-	emit exportPOV(qstr);
+	tmpdir = prefs.getTempPath();
 
-//	qstr = "rm -f " + tmp + "/tmp.ppm";
-//	system(qstr);
+	tmpname.sprintf("tmp%i",rand());
+	
+	qstr = tmpdir + "/" + tmpname + ".pov";
+	
+	viewnum = worldptr->getViewPosition(this);
+	if(viewnum < 0)
+		return;
+			
+	emit report(i18n("Writing renderfile"));
+	emit exportPOV(qstr,viewnum);
 
 	proc.clearArguments();
 
-	proc << pref.getRenderProgram();
+	proc << prefs.getRenderProgram();
 	sprintf(str,"+W%i",width());
 	proc << str;
 	sprintf(str,"+H%i",height());
-	proc << str << "-GA";
-	qstr = "-O" + tmp + "/tmp.ppm";
+	proc << str;
+//	proc << "-GA";
+	qstr = "-O" + tmpdir + "/" + tmpname + ".ppm";
 	proc << qstr;
-	qstr = "-I" + tmp + "/tmp.pov";
+	qstr = "-I" + tmpdir + "/" + tmpname + ".pov";
 	proc << qstr << "+FP";
 	qstr.sprintf("-Q%i",quality);
 	proc << qstr;
@@ -883,64 +1010,114 @@ void		ViewWidget::startRendering(int quality)
 	ppmfp = 0;
 	lc = 0;
 
-//	sleep(1);
-	startTimer(50);
+	startTimer(150);
+	
+	qstr = QString(i18n("Start rendering with")) + " ";
+	qstr += prefs.getRenderProgram();
+	emit report(qstr);
 }
 
 void		ViewWidget::stopRendering()
 {
-	QString		qstr,tmp;
-	Preferences	pref;
+	QString		qstr;
 	
-	if(!ppmfp) return;
-
-	qstr = "Ready";
+	if(!ppmfp)
+		return;
+		
+	qstr = i18n("Ready");
 	emit report(qstr);
 	
 	killTimers();
 	proc.kill();
-	fclose(ppmfp);
-	flush();
+	if(ppmfp)
+		fclose(ppmfp);
 	ppmfp = 0;
+	flush();
 	
-
-	killTimers();
-	
-	tmp = pref.getTempPath();
-	qstr = "rm -f " + tmp + "/tmp.ppm";
+	qstr = "rm -f " + tmpdir + "/" + tmpname + ".ppm";
 	system(qstr);
+	qstr = "rm -f " + tmpdir + "/" + tmpname + ".pov";
+	system(qstr);
+	
+	tcounter = -1;
+	
+	emit stoppedRendering();
 }
 
 void		ViewWidget::timerEvent(QTimerEvent*)
 {
-	unsigned char	buffer[3 * 1024];
-	int		i,t,r,g,b,p;
-	Preferences	pref;
-	QString		qstr,tmpstr;
+	QString		tmpstr,qstr;
 
+	if(!proc.isRunning())
+	{
+		if(proc.normalExit())
+		{
+#ifdef DEBUG
+			printf("Renderer exitstatus:%i\n",proc.exitStatus());
+#endif
+		}
+		else
+		{
+			QString		errormsg;
+
+			errormsg = i18n("Renderer killed by foreign process!");
+			emit report(errormsg);
+		}
+		killTimers();
+		proc.kill();
+		tcounter = -1;
+
+		if(ppmfp != 0)
+			readImage();
+		// Attention! readImage could close the file
+		if(ppmfp != 0)
+		{
+			fclose(ppmfp);
+			ppmfp = 0;
+		}
+		flush();
+			
+		return;	
+	}
 	if(ppmfp == 0)
 	{
-		qstr = pref.getTempPath();
 
 		tcounter++;
-		// 400 * 50ms = 20000ms = 20sec timeout
-		if(tcounter > 400)
+		// 33 * 150ms = 4950ms = ~5sec
+		if(tcounter == 33)
 		{
-			stopRendering();
+			emit report(i18n("Renderer is parsing"));
+		}
+		// 800 * 150ms = 120000ms = 120sec timeout
+		if(tcounter > 800)
+		{
+			emit report(i18n("Stopped rendering becauseof timeout"));
+
+			killTimers();
+			proc.kill();
+
 			return;
 		}
-		ppmfp = fopen(qstr + "/tmp.ppm","rb");
-		if(ppmfp == 0) return;
-//		sleep(1);
-//		printf("File opened\n");
+		
+		ppmfp = fopen(tmpdir + "/" + tmpname + ".ppm","rb");
+		
+		if(ppmfp == 0)
+			return;
 		fseek(ppmfp,0,SEEK_SET);
 	}
+	readImage();
+}
+
+void	ViewWidget::readImage()
+{
+	unsigned char	buffer[3 * 1024];
+	int		i,t,r,g,b,p;
+	QString		tmpstr,qstr;
+	
 	if(lc < 3)
 	{
-//		printf("Try new line\n");
 		p = ftell(ppmfp);
 		fseek(ppmfp,0,SEEK_END);
-//		printf("diff %i - %i = %i\n",ftell(ppmfp),p,ftell(ppmfp) - p);
 		if(ftell(ppmfp) <= p)
 		{
 			fseek(ppmfp,p,SEEK_SET);
@@ -951,11 +1128,9 @@ void		ViewWidget::timerEvent(QTimerEvent*)
 		do
 		{
 			fread(buffer,1,1,ppmfp);
-//			printf("%c",buffer[0]);
 		}
 		while(buffer[0] != '\n');
 		lc++;
-//		printf("\nNew Line\n");
 		return;
 	}
 
@@ -968,60 +1143,289 @@ void		ViewWidget::timerEvent(QTimerEvent*)
 	}
 	fseek(ppmfp,p,SEEK_SET);
 	
-	i = fread(buffer,3,1024,ppmfp);
-
-	for(t = 0;t < i;t++)
+	while((i = fread(buffer,3,1024,ppmfp)) > 0)
 	{
-		r = buffer[t * 3];
-		g = buffer[t * 3 + 1];
-		b = buffer[t * 3 + 2];
-		qp->setPen(QColor(r,g,b));
-		qp->drawPoint(x++,y);
-		if(x >= width())
+
+		for(t = 0;t < i;t++)
 		{
-			x = 0;
-			y++;
-			if(y >= height())
+			r = buffer[t * 3];
+			g = buffer[t * 3 + 1];
+			b = buffer[t * 3 + 2];
+			qp->setPen(QColor(r,g,b));
+			qp->drawPoint(x++,y);
+			if(x >= width())
 			{
-				stopRendering();
-				return;
+				x = 0;
+				y++;
+				if(y >= height())
+				{
+					emit finishedRendering();
+					stopRendering();
+					
+					return;
+				}
 			}
 		}
 	}
 	
-	i = fread(buffer,3,1024,ppmfp);
-
-	for(t = 0;t < i;t++)
-	{
-		r = buffer[t * 3];
-		g = buffer[t * 3 + 1];
-		b = buffer[t * 3 + 2];
-		qp->setPen(QColor(r,g,b));
-		qp->drawPoint(x++,y);
-		if(x >= width())
-		{
-			x = 0;
-			y++;
-			if(y >= height())
-			{
-				stopRendering();
-				return;
-			}
-		}
-	}
 	flush();
 
 	tmpstr.setNum(((x + y * width()) * 100) / (width() * height()));
+	qstr = (QString)i18n("Rendered") + " ";
+	qstr += tmpstr + "%";
+	emit report(qstr);
+}
+
+#endif
+
+#if (USE_PVM == 1)
+// PVM-Version !!
+void		ViewWidget::startRendering(int quality)
+{
+	QString		qstr,tmp;
+	int		viewnum;
+
+
+	int length,i,j,k;
+	FILE 	*file;
+	char *data;
+
+	ClusterInfo *ci;
+	Preferences prefs;
+
+	if (pvm_on==true) stopRendering();
+	pvm_on=true;
+
+	// Is PVM running on the localhost ???
+
+	if ((mytid=pvm_mytid())<0) {
+		qstr = i18n("Couldn't start PVM....Daemon not running ?");
+		emit report(qstr);
+		return;
+	}
+
+	// export the POV-File
+	
+	tmp = prefs.getTempPath();
+	tmpname.sprintf("tmp%i",rand());
+	qstr = tmp + "/" + tmpname + ".pov";
+	viewnum = worldptr->getViewPosition(this);
+	if(viewnum < 0) return;
+	emit exportPOV(qstr,viewnum);
+	file=fopen(qstr,"r");
+	if (file) {
+		fseek(file,0,SEEK_END);
+		length=ftell(file);
+		fseek(file,0,SEEK_SET);
+		data=(char *)malloc(length);
+		if (data) {
+			fread(data,1,length,file);
+		} else return;
+		fclose(file);
+	} else return;
+
+	// Add hosts to the PVM-Config
+
+	for (i=0;i<prefs.numClusterHosts();i++) {
+		char *host;
+		host=strdup(prefs.recentClusterHost(i));
+
+		j=pvm_addhosts(&host,1,&k);				
+		if (host) free(host);
+	}
+
+        // Get info
+	
+	pvm_config(&nprocessors,(int*)0,&pi);
+	
+
+	for (i=0;i<nprocessors;i++) {
+	     	ci=new ClusterInfo;
+		ci->pinfo=&pi[i];   // Clustername, speed(?), id
+		ci->x1=0;
+		ci->y1=0;
+		ci->x2=0;
+		ci->y2=0;
+		ci->timePline=0.0;
+		clusters.append(ci);
+	}
+
+	pixel=0;
+
+	
+	for (i=0;i<nprocessors;i++) {
+		int xywhq[9];
+		int tile=height()/nprocessors;
+		pvmhostinfo *hi=clusters[i]->pinfo;
+		printf("%d %s\n",i,hi->hi_name);
+		pvm_spawn("k4ded",(char**)0,0,"",1,&hi->hi_tid );
+
+		xywhq[0]=length;		
+		clusters[i]->x1=xywhq[1]=0;
+		clusters[i]->y1=xywhq[2]=tile*i;  // start-y
+		clusters[i]->x2=xywhq[3]=width();
+		clusters[i]->y2=xywhq[4]=tile*i+tile;  // end-y
+		xywhq[5]=width();
+		xywhq[6]=height();
+		xywhq[7]=quality;
+		pvm_initsend(PvmDataDefault);
+		pvm_pkint(xywhq,8,1);
+		pvm_send(hi->hi_tid,1);
+
+		pvm_initsend(PvmDataDefault);
+		pvm_packf("%+ %*c", PvmDataDefault,length,data);
+		pvm_send(hi->hi_tid,2);
+	}
+	pixel=width()*height();
+	pixel_togo=pixel;
+
+	if (data) free(data);
+	
+	killTimers();
+
+	startTimer(150);
+
+	tmp.setNum(nprocessors);
+	qstr = "PVM-Rendering started on ";
+	qstr += tmp + " Processors";
+	emit report(qstr);
+}
+
+void		ViewWidget::stopRendering()
+{
+	QString		qstr;
+        Preferences	prefs;
+	int i,j,k;	
+	
+	emit report(qstr);
+
+	killTimers();
+	flush();
+
+	killTimers();
+	
+	tcounter = -1;
+	if (pvm_on==true) {
+		for (i=0;i<clusters.length();i++) {
+			char *host=clusters[i]->pinfo->hi_name;
+			j=pvm_delhosts(&host,1,&k);				
+			printf("Removed %s  %d\n",host,j );
+		}
+		pvm_exit();
+		qstr = "PVM-Rendering stopped.";
+		emit report(qstr);
+		while (clusters.length()>0) {
+			delete clusters[0];
+			clusters.deleteAt(0);
+		}
+		nprocessors=0;
+		pvm_on=false;
+	}
+
+	emit stoppedRendering();
+}
+
+void		ViewWidget::timerEvent(QTimerEvent*)
+{
+	QString		qstr,tmpstr;
+	int id,msg_tag,msg_tid;
+	int x1,y1,x2,y2,length,time,j,i;
+	int t,r,g,b,lc;
+	int tid;	
+	char *data;
+	char *hostname;
+	unsigned char *buffer;
+
+
+	if (!(id=pvm_probe(-1,20))) return;
+
+	//killTimers();
+
+	pvm_bufinfo(id,&j,&msg_tag,&msg_tid);
+
+	pvm_recv(msg_tid,20);
+	//pvm_bufinfo(mid,(int*)0,(int*)0,&slaveid);
+	pvm_unpackf("%d %d %d %d %d %d",&length,&time,&x1,&y1,&x2,&y2);
+
+	for (i=0;i<nprocessors;i++) {
+		if (msg_tid==clusters[i]->pinfo->hi_tid) hostname=clusters[i]->pinfo->hi_name;
+		//printf("%d\n",clusters[i]->pinfo->hi_tid);
+	}
+
+	printf("recv from %s |%d: %d %d %d %d  (time:%d sec)\n",hostname,msg_tid,x1,y1,x2,y2,time);
+
+	data=(char *)malloc(length);
+	//printf("rec %d bytes from %d\n",length,slaveid);
+
+	pvm_recv(msg_tid,21);
+	pvm_bufinfo(pvm_getrbuf(),&j,(int*)0,&tid);
+	pvm_upkbyte(data,length,1);			
+
+
+	//Overread PPM-Header
+	lc=0;i=0;
+	while ((lc<3)&&(i<length)) {
+		if (((*(data+i))==10)||((*(data+i))==13)) lc++;
+		i++;
+	}
+	if (lc<3) {return;}
+	buffer=(unsigned char*)(data+i);
+	
+	x=x1;
+	y=y1;
+        pixel_togo-=(x2-x1)*(y2-y1);
+
+	t=0;
+	for (y=y1;y<y2;y++) {
+		for (x=x1;x<x2;x++) {
+			r = buffer[t*3 ];
+			g = buffer[t*3 + 1];
+			b = buffer[t*3 + 2];
+			qp->setPen(QColor(r,g,b));
+			qp->drawPoint(x,y);
+			t++;
+		}
+	}
+
+	flush();
+	//startTimer(150);
+
+	if (pixel_togo<=0) {
+		emit finishedRendering();
+		stopRendering();
+	}
+
+	tmpstr.setNum((( (pixel-pixel_togo) * 100) / (pixel)));
 	qstr = "Rendered ";
 	qstr += tmpstr + "%";
 	emit report(qstr);
 }
 
+
+#endif
+
+
 int		ViewWidget::setName(const char *n)
 {
 	return view::setName(n);
 }
 
+void		ViewWidget::saveViewImage(QString fname,QString fmt)
+{
+	QString		qstr;
+	
+	if(qpm->save(fname,fmt))
+	{
+		qstr = "Saved " + fname + " as " + fmt;
+		emit report(qstr);
+	}
+	else
+	{
+		qstr = "Failed saving " + fname + " as " + fmt;
+		emit report(qstr);
+	}
+}
+
 void		ViewWidget::reconfigure()
 {
 	configure();
@@ -1030,7 +1434,7 @@ void		ViewWidget::reconfigure()
 
 void		ViewWidget::configure()
 {
-	Preferences	prefs;
+	Preferences prefs;
 
 	backcolor = prefs.getBackgroundColor(); 
 	drawcolor = prefs.getLinesColor(); 
@@ -1065,5 +1469,20 @@ void		ViewWidget::configure()
 	grid_zy = prefs.getGridZY();
 }
 
+void		ViewWidget::setViewObject(base *b)
+{
+	setSubObject(b);
+	draw();
+}
+
+void		ViewWidget::resize()
+{
+	resize(w,h);
+}
+
+void		ViewWidget::resize(int w,int h)
+{
+	QFrame::resize(w,h);
+}
 
 
