@@ -24,14 +24,15 @@
 ** worldtree.cpp
 */
 
-#include <klocale.h>
-
-
-#define stloc(a)	((char*)klocale->translate((a)))
-
+#include <kapp.h>
 #include <qpoint.h>
 
 #include "../dialogs/dialogs.h"
+#include "../dialogs/transformdialog.h"
+#include "../dialogs/map2mesh.h"
+#include "../dialogs/recursivtree.h"
+#include "../dialogs/forestgen.h"
+#include "../dialogs/df3.h"
 #include "../misc/preferences.h"
 
 #include "worldtree.h"
@@ -41,6 +42,7 @@
 
 #include "objects.h"
 
+
 #include "icons/tree/x.xpm"
 
 #include "icons/tree/world.xpm"
@@ -64,6 +66,8 @@
 #include "icons/tree/spotlight.xpm"
 #include "icons/tree/arealight.xpm"
 #include "icons/tree/camera2.xpm"
+#include "icons/tree/ear.xpm"
+#include "icons/tree/view.xpm"
 
 #include "icons/menu/editcopy.xpm"
 #include "icons/menu/editpaste.xpm"
@@ -82,7 +86,7 @@ WorldTree::WorldTree(QWidget *p,char *name) :
 
 	createWidgets();
 
-	for(t = 0;t < 50;t++) 
+	for(t = 0;t < 50;t++)
 	{
 		if(t == NUM_WORLD) pixmap[t] = new QPixmap((const char**)world_xpm);
 		else if(t == NUM_OBJ) pixmap[t] = new QPixmap((const char**)obj_xpm);
@@ -105,6 +109,8 @@ WorldTree::WorldTree(QWidget *p,char *name) :
 		else if(t == NUM_AREALIGHT) pixmap[t] = new QPixmap((const char**)arealight_xpm);
 		else if(t == NUM_LIGHTPOINT) pixmap[t] = new QPixmap((const char**)lightpoint_xpm);
 		else if(t == NUM_CAMERA) pixmap[t] = new QPixmap((const char**)camera_xpm);
+		else if(t == NUM_EAR) pixmap[t] = new QPixmap((const char**)ear_xpm);
+		else if(t == NUM_VIEW) pixmap[t] = new QPixmap((const char**)view_xpm);
 		else pixmap[t] = new QPixmap((const char**)x_xpm);
 	}
 
@@ -114,15 +120,25 @@ WorldTree::WorldTree(QWidget *p,char *name) :
 
 WorldTree::~WorldTree()
 {
-	if(worldptr) delete worldptr;
+	int		t;
+	
+	if(worldptr)
+		delete worldptr;
 
-	if(capptr) delete capptr;
+	if(capptr)
+		delete capptr;
+		
+	for(t = 0;t < 50;t++)
+	{
+		if(pixmap[t])
+			delete pixmap[t];
+	}
 }
 
 void	WorldTree::createWidgets()
 {
-	addColumn("Object tree",150);
-	addColumn("Texture",120);
+	addColumn(i18n("Objecttree"),150);
+	addColumn(i18n("Texture"),120);
 	addColumn("Sort",1);
 	setColumnWidthMode(0,QListView::Maximum);
 	setColumnWidthMode(1,QListView::Manual);
@@ -182,6 +198,25 @@ void	WorldTree::createTree(base *b,QListViewItem *pit,int num)
 	sprintf(str,"%i",num);
 	li->setText(2,str);
 
+	if(b->getType() == NUM_WORLD)
+	{
+		TreeListItem	*vi;
+		view		*v;
+		
+		viewlistitem = new TreeListItem(li,i18n("Viewlist"),20);
+		viewlistitem->setHeight(20);
+		viewlistitem->setPixmap(0,*pixmap[NUM_VIEW]);
+		viewlistitem->setExpandable(TRUE);
+		viewlistitem->setText(2," ");
+		for(t = 0;t < ((world*)b)->getNumViews();t++)
+		{
+			v = ((world*)b)->getView(t);
+			vi = new TreeListItem(viewlistitem,v->getName(),20);
+			vi->setHeight(20);
+			vi->setPixmap(0,*pixmap[NUM_VIEW]);
+		}
+	}
+	
 	b->getChildren(&children);
 	if(!children)	return;
 
@@ -194,6 +229,25 @@ void	WorldTree::createTree(base *b,QListViewItem *pit,int num)
 	if(children) free(children);
 }
 
+void	WorldTree::createTree(base *b,QListViewItem *it)
+{
+	base		**children = 0;
+	int		t;
+
+	if(!b || !it) return;
+
+ 	b->getChildren(&children);
+	if(!children)	return;
+
+	t = 0;
+	while(children[t])
+	{
+		createTree(children[t],it,t);
+		t++;
+	}
+	if(children) free(children);
+}
+
 void	WorldTree::layout()
 {
 	int	w,h,cw;
@@ -215,19 +269,19 @@ void	WorldTree::draw()
 	repaint();
 }
 
-int	WorldTree::exportPOV(const char *filename,view *v,int anim)
+int	WorldTree::exportPOV(const char *filename,int n,int anim)
 {
 	FILE	*fp; 
 	int	error;
-
-	fp = fopen(filename,"wb"); 
+	
+	fp = fopen(filename,"wb");
  
 	if(fp == 0) return -3;
 
-//	error = v->exportPOV(fp,4);
-	error = worldptr->exportPOV(fp,0,4,anim);
+	error = worldptr->exportPOV(n,fp,0,4,anim);
 
-	fclose(fp); 
+	fflush(fp);
+	fclose(fp);
 
 	return error;
 }
@@ -272,9 +326,11 @@ int	WorldTree::load(const char *filename)
 
 int	WorldTree::empty()
 {
+	emit emptyWorld();
 
 	if(worldptr)
 		delete worldptr;
+		
 	worldptr = new world("World");
 	createTree();
 	
@@ -285,8 +341,25 @@ int	WorldTree::empty()
 	return 0;
 }
 
+int	WorldTree::empty(QListViewItem *pit)
+{
+        QListViewItem *lit;
+
+        while((lit = pit->firstChild()))
+        {
+        	pit->removeItem(lit);
+        }
+	
+	emit redraw();
+
+	return 0;
+}
+
 void	WorldTree::setWorld(world *nw)
 {
+	if(worldptr == nw)
+		return;
+		
 	worldptr = nw;
 	createTree();
 
@@ -309,11 +382,204 @@ void	WorldTree::dump()
 	worldptr->dumpNames(0,4);
 }
 
+int	WorldTree::map2Mesh()
+{
+	Map2MeshDialog	*m2md;
+	QListViewItem	*i;
+	base		*b;
+	smesh		*m;
+	
+	if(!(i = currentItem()))
+	{
+		return -1;
+	}
+
+	if(!(b = worldptr->searchName((char*)i->text(0))))
+	{
+		return -2;
+	}
+	
+	if(b->getType() == NUM_SMESH || b->getType() == NUM_CMESH)
+	{
+		m = (smesh*)b;
+	}
+	else
+	{
+		if(addX((m = new smesh(0,"Mesh"))) != 0)
+			return -3;
+	}
+
+	m2md = new Map2MeshDialog();
+	
+	m2md->setObject(m);
+	
+	m2md->show();
+	
+	printf("result = %i\n",m2md->result());
+	
+	if(m2md->result() == QDialog::Accepted)
+	{
+		m2md->getObject();
+	}
+	
+	delete m2md;
+	
+	emit redraw();
+	emit changedModel();
+	
+	return 0;
+}
+
+int	WorldTree::recursivTree()
+{
+	RecursivTreeDialog	*rtd;
+	QListViewItem		*i;
+	base			*b;
+	obj			*o;
+	
+	if(!(i = currentItem()))
+	{
+		return -1;
+	}
+
+	if(!(b = worldptr->searchName((char*)i->text(0))))
+	{
+		return -2;
+	}
+	
+	if(b->getType() == NUM_OBJ)
+	{
+		o = (obj*)b;
+	}
+	else
+	{
+		if(addX((o = new obj(0,"Tree"))) != 0)
+			return -3;
+		o->rotate(Vector3(0,0,PI / 2));
+	}
+
+	o->setFlag(base::HIDE);
+	
+	rtd = new RecursivTreeDialog();
+
+	connect(rtd,SIGNAL(report(QString)),this,SIGNAL(report(QString)));
+	
+	rtd->setObject(o);
+	
+	rtd->show();
+	
+
+	if(rtd->result() == QDialog::Accepted)
+	{
+		rtd->getObject();
+		if(b->getType() == NUM_OBJ)
+		{
+			empty(i);
+			createTree(o,i);
+		}
+		else
+		{
+			createTree(o,currentItem());
+		}
+	}
+	
+	delete rtd;
+	
+	emit redraw();
+	emit changedModel();
+	
+	return 0;
+}
+
+int	WorldTree::generateForest()
+{
+	ForestGeneratorDialog	*fgd;
+	QListViewItem		*i;
+	base			*b;
+	obj			*o;
+	
+	if(!(i = currentItem()))
+	{
+		return -1;
+	}
+
+	if(!(b = worldptr->searchName((char*)i->text(0))))
+	{
+		return -2;
+	}
+	
+	if(b->getType() == NUM_OBJ)
+	{
+		o = (obj*)b;
+	}
+	else
+	{
+		if(addX((o = new obj(0,"Forest"))) != 0)
+			return -3;
+	}
+
+	o->setFlag(base::HIDE);
+	
+	fgd = new ForestGeneratorDialog(pixmap,o);
+
+	connect(fgd,SIGNAL(report(QString)),this,SIGNAL(report(QString)));
+	
+	fgd->show();
+	
+	if(fgd->result() == QDialog::Accepted)
+	{
+		fgd->getObject();
+		if(b->getType() == NUM_OBJ)
+		{
+			empty(i);
+			createTree(o,i);
+		}
+		else
+		{
+			createTree(o,currentItem());
+		}
+	}
+	
+	delete fgd;
+	
+	emit redraw();
+	emit changedModel();
+	
+	return 0;
+}
+
+int	WorldTree::readDF3()
+{
+	DF3Dialog	*df3d;
+	QListViewItem	*i;
+	base		*b;
+	
+	if(!(i = currentItem()))
+	{
+		return -1;
+	}
+
+	if(!(b = worldptr->searchName((char*)i->text(0))))
+	{
+		return -2;
+	}
+	
+	df3d = new DF3Dialog(b);
+	
+	df3d->show();
+	
+	delete df3d;
+	
+	emit redraw();
+	emit changedModel();
+	
+	return 0;
+}
+
 int	WorldTree::addX(base *child)
 {
 	QListViewItem	*i;
 	QListViewItem	*li;
-	QListViewItem	*ci;
 	base		*b;
 	char		*n;
 	int		num;
@@ -329,17 +595,20 @@ int	WorldTree::addX(base *child)
 		return -4;
 	}
 	child->setName(n);
+	free(n);
 
 	if(!(i = currentItem()))
 	{
-		delete child;
-		return -3;
+		b = worldptr;
+		i = firstChild();
 	}
-
-	if(!(b = worldptr->searchName((char*)i->text(0))))
+	else
 	{
-		delete child;
-		return -2;
+		if(!(b = worldptr->searchName((char*)i->text(0))))
+		{
+			delete child;
+			return -2;
+		}
 	}
 
 	if(child->addToParent(b) != 0)
@@ -358,22 +627,18 @@ int	WorldTree::addX(base *child)
 		child->setFlag(base::DRAW_AXIS,0);
 	}
 
-	if((ci = i->firstChild()))
-	{
-		while(ci->nextSibling())
-			ci = ci->nextSibling();
-	}
-	if(ci)
-		num = atoi(ci->text(2)) + 1;
-	else
-		num = 0;
+	num = b->numChildren();
+	
 	li = new TreeListItem(i,child->getName(),20);
 	li->setPixmap(0,*pixmap[child->getType()]);
 	sprintf(str,"%i",num);
 	li->setText(2,str);
 	i->setOpen(TRUE);
+	setCurrentItem(li);
+ 	setSelected(li,true);
 
 	emit redraw();
+	emit changedModel();
 
 	return 0;
 }
@@ -462,6 +727,11 @@ void	WorldTree::addText()
 	delete td;
 }
 
+void	WorldTree::addScript()
+{
+	addX(new script(0,"Script"));
+}
+
 void	WorldTree::addCone()
 {
 	addX(new cone(0,"Cone"));
@@ -502,6 +772,39 @@ void	WorldTree::addCamera()
 	addX(new camera(0,"Camera"));
 }
 
+void	WorldTree::addEar()
+{
+	addX(new ear(0,"Ear",0));
+}
+
+
+
+world	*WorldTree::getWorld()
+{
+	return worldptr;
+}
+
+base	*WorldTree::getSelectedObject()
+{
+	QListViewItem	*li;
+	base		*b;
+		
+	if((li = currentItem()))
+	{
+		b = worldptr->searchName((char*)li->text(0));
+		if(!b)
+		{
+			return 0;
+		}
+		else
+		{
+			return b;
+		}
+	}
+		
+	return 0;
+}
+
 void	WorldTree::popupMenu(QListViewItem *li,int x,int y)
 {
 	base		*b;
@@ -518,87 +821,89 @@ void	WorldTree::popupMenu(QListViewItem *li,int x,int y)
 	}
 
 	pm = new QPopupMenu();
-	pm->insertItem("Rename",1);
+	pm->insertItem(i18n("Rename"),1);
 
 	switch(b->getType())
 	{
+		case NUM_VIEW:
+		break;
 		case NUM_WORLD:
-			pm->insertItem(QPixmap((const char**)editpaste_xpm),"Paste",21);
+			pm->insertItem(QPixmap((const char**)editpaste_xpm),i18n("Paste"),21);
 			pm->setItemEnabled(21,!(!capptr));
-			pm->insertItem(QPixmap((const char**)edit_xpm),"Edit Animscript Main",27);
-			pm->insertItem(QPixmap((const char**)world_xpm),"World",29);
+			pm->insertItem(QPixmap((const char**)edit_xpm),i18n("Edit Animscript Main"),27);
+			pm->insertItem(QPixmap((const char**)world_xpm),i18n("World"),29);
 		break;
 		case NUM_REF:
-			pm->insertItem(QPixmap((const char**)editcut_xpm),"Cut",20);
-			pm->insertItem(QPixmap((const char**)editcopy_xpm),"Copy",22);
-			pm->insertItem(QPixmap((const char**)editpaste_xpm),"Paste",21);
+			pm->insertItem(QPixmap((const char**)editcut_xpm),i18n("Cut"),20);
+			pm->insertItem(QPixmap((const char**)editcopy_xpm),i18n("Copy"),22);
+			pm->insertItem(QPixmap((const char**)editpaste_xpm),i18n("Paste"),21);
 			pm->setItemEnabled(21,!(!capptr));		
 			pm->insertSeparator();
-			id = pm->insertItem("Hide",25);
+			id = pm->insertItem(i18n("Hide"),25);
 			if(b->isFlag(base::HIDE))
 				pm->setItemChecked(id,!0);
-			id = pm->insertItem("BBox",26);
+			id = pm->insertItem(i18n("BBox"),26);
 			if(b->isFlag(base::DRAW_BOUNDINGBOX))
 				pm->setItemChecked(id,!0);
-			id = pm->insertItem("Axis",30);
+			id = pm->insertItem(i18n("Axis"),30);
 			if(b->isFlag(base::DRAW_AXIS))
 				pm->setItemChecked(id,!0);
-			pm->insertItem(QPixmap((const char**)edit_xpm),"Edit Animscript",28);
+			pm->insertItem(QPixmap((const char**)edit_xpm),i18n("Edit Animscript"),28);
 		break;
 		case NUM_OBJ:
 		case NUM_CSG:
 		case NUM_BLOB:
-			pm->insertItem(QPixmap((const char**)editcut_xpm),"Cut",20);
-			pm->insertItem(QPixmap((const char**)editcopy_xpm),"Copy",22);
-			pm->insertItem(QPixmap((const char**)editpaste_xpm),"Paste",21);
+			pm->insertItem(QPixmap((const char**)editcut_xpm),i18n("Cut"),20);
+			pm->insertItem(QPixmap((const char**)editcopy_xpm),i18n("Copy"),22);
+			pm->insertItem(QPixmap((const char**)editpaste_xpm),i18n("Paste"),21);
 			pm->setItemEnabled(21,!(!capptr));		
 			pm->insertSeparator();
-			id = pm->insertItem("Hide",25);
+			id = pm->insertItem(i18n("Hide"),25);
 			if(b->isFlag(base::HIDE))
 				pm->setItemChecked(id,!0);
-			id = pm->insertItem("BBox",26);
+			id = pm->insertItem(i18n("BBox"),26);
 			if(b->isFlag(base::DRAW_BOUNDINGBOX))
 				pm->setItemChecked(id,!0);
-			id = pm->insertItem("Axis",30);
+			id = pm->insertItem(i18n("Axis"),30);
 			if(b->isFlag(base::DRAW_AXIS))
 				pm->setItemChecked(id,!0);
 			pm->insertSeparator();
-			id = pm->insertItem("Hollow on",32);
+			id = pm->insertItem(i18n("Hollow on"),32);
 			if(b->isFlag(base::HOLLOW_ON))
 				pm->setItemChecked(id,!0);
-			id = pm->insertItem("Hollow off",33);
+			id = pm->insertItem(i18n("Hollow off"),33);
 			if(b->isFlag(base::HOLLOW_OFF))
 				pm->setItemChecked(id,!0);
 			pm->insertSeparator();
-			pm->insertItem(QPixmap((const char**)transform_xpm),"Dim",3);
-			pm->insertItem(QPixmap((const char**)edit_xpm),"Edit Animscript",28);
+			pm->insertItem(QPixmap((const char**)transform_xpm),i18n("Transform"),3);
+			pm->insertItem(QPixmap((const char**)edit_xpm),i18n("Edit Animscript"),28);
 		break;
 		default:
-			pm->insertItem(QPixmap((const char**)editcut_xpm),"Cut",20);
-			pm->insertItem(QPixmap((const char**)editcopy_xpm),"Copy",22);
+			pm->insertItem(QPixmap((const char**)editcut_xpm),i18n("Cut"),20);
+			pm->insertItem(QPixmap((const char**)editcopy_xpm),i18n("Copy"),22);
 			pm->insertSeparator();
-			id = pm->insertItem("Hide",25);
+			id = pm->insertItem(i18n("Hide"),25);
 			if(b->isFlag(base::HIDE))
 				pm->setItemChecked(id,!0);
 			if(b->isFinite())
 			{
-				id = pm->insertItem("BBox",26);
+				id = pm->insertItem(i18n("BBox"),26);
 				if(b->isFlag(base::DRAW_BOUNDINGBOX))
 					pm->setItemChecked(id,!0);
 			}
-			id = pm->insertItem("Axis",30);
+			id = pm->insertItem(i18n("Axis"),30);
 			if(b->isFlag(base::DRAW_AXIS))
 				pm->setItemChecked(id,!0);
 			pm->insertSeparator();
-			id = pm->insertItem("Hollow on",32);
+			id = pm->insertItem(i18n("Hollow on"),32);
 			if(b->isFlag(base::HOLLOW_ON))
 				pm->setItemChecked(id,!0);
-			id = pm->insertItem("Hollow off",33);
+			id = pm->insertItem(i18n("Hollow off"),33);
 			if(b->isFlag(base::HOLLOW_OFF))
 				pm->setItemChecked(id,!0);
 			pm->insertSeparator();
-			pm->insertItem(QPixmap((const char**)transform_xpm),"Dim",3);
-			pm->insertItem(QPixmap((const char**)edit_xpm),"Edit Animscript",28);
+			pm->insertItem(QPixmap((const char**)transform_xpm),i18n("Transform"),3);
+			pm->insertItem(QPixmap((const char**)edit_xpm),i18n("Edit Animscript"),28);
 	}
 
 
@@ -606,65 +911,79 @@ void	WorldTree::popupMenu(QListViewItem *li,int x,int y)
 	{
 		case NUM_CONE:
 			pm->insertSeparator();
-			pm->insertItem("Cone",4);
+			pm->insertItem(i18n("Cone"),4);
 		break;
 		case NUM_TORUS:
 			pm->insertSeparator();
-			pm->insertItem("Torus",5);
+			pm->insertItem(i18n("Torus"),5);
 		break;
 		case NUM_REFPTR:
 			pm->insertSeparator();
-			pm->insertItem("Reference",6);
+			pm->insertItem(i18n("Reference"),6);
 		break;
 		case NUM_CSG:
 			pm->insertSeparator();
-			id = pm->insertItem("Union",11);
+			id = pm->insertItem(i18n("Union"),11);
 			if(((csg*)b)->getCSGType() == csg::UNION)
 				pm->setItemChecked(id,!0);
-			id = pm->insertItem("Merge",12);
+			id = pm->insertItem(i18n("Merge"),12);
 			if(((csg*)b)->getCSGType() == csg::MERGE)
 				pm->setItemChecked(id,!0);
-			id = pm->insertItem("Intersection",13);
+			id = pm->insertItem(i18n("Intersection"),13);
 			if(((csg*)b)->getCSGType() == csg::INTERSECTION)
 				pm->setItemChecked(id,!0);
-			id = pm->insertItem("Difference",14);
+			id = pm->insertItem(i18n("Difference"),14);
 			if(((csg*)b)->getCSGType() == csg::DIFFERENCE)
 				pm->setItemChecked(id,!0);
-			id = pm->insertItem("Invert",15);
+			id = pm->insertItem(i18n("Invert"),15);
 			if(((csg*)b)->getCSGType() == csg::INVERT)
 				pm->setItemChecked(id,!0);
 		break;
 		case NUM_BLOB:
 			pm->insertSeparator();
-			pm->insertItem(QPixmap((const char**)blob_xpm),"Blob",8);
+			pm->insertItem(QPixmap((const char**)blob_xpm),i18n("Blob"),8);
 		break;
 		case NUM_TEXT:
 			pm->insertSeparator();
-			pm->insertItem(QPixmap((const char**)text_xpm),"Text",19);
+			pm->insertItem(QPixmap((const char**)text_xpm),i18n("Text"),19);
+		break;
+		case NUM_SCRIPT:
+			pm->insertSeparator();
+			pm->insertItem(QPixmap((const char**)script_xpm),i18n("Script"),38);
 		break;
 		case NUM_TRIANGLE:
 			pm->insertSeparator();
-			pm->insertItem(QPixmap((const char**)triangle_xpm),"Triangle",9);
+			pm->insertItem(QPixmap((const char**)triangle_xpm),i18n("Triangle"),9);
 		break;
 		case NUM_HEIGHTFIELD:
 			pm->insertSeparator();
-			pm->insertItem(QPixmap((const char**)heightfield_xpm),"Heightfield",10);
+			pm->insertItem(QPixmap((const char**)heightfield_xpm),i18n("Heightfield"),10);
 		break;
 		case NUM_LIGHTPOINT:
 			pm->insertSeparator();
-			pm->insertItem(QPixmap((const char**)lightpoint_xpm),"Lightpoint",16);
+			pm->insertItem(QPixmap((const char**)lightpoint_xpm),i18n("Lightpoint"),16);
 		break;
 		case NUM_SPOTLIGHT:
 			pm->insertSeparator();
-			pm->insertItem(QPixmap((const char**)spotlight_xpm),"Spotlight",17);
+			pm->insertItem(QPixmap((const char**)spotlight_xpm),i18n("Spotlight"),17);
 		break;
 		case NUM_AREALIGHT:
 			pm->insertSeparator();
-			pm->insertItem(QPixmap((const char**)arealight_xpm),"Arealight",18);
+			pm->insertItem(QPixmap((const char**)arealight_xpm),i18n("Arealight"),18);
 		break;
 		case NUM_CAMERA:
 			pm->insertSeparator();
-			pm->insertItem(QPixmap((const char**)camera_xpm),"Camera",31);
+			pm->insertItem(QPixmap((const char**)camera_xpm),i18n("Camera"),31);
+		break;
+		case NUM_EAR:
+			pm->insertSeparator();
+			pm->insertItem(QPixmap((const char**)ear_xpm),i18n("Ear"),37);
+		break;
+		case NUM_CMESH:
+			pm->insertSeparator();
+			pm->insertItem(QPixmap((const char**)triangle_xpm),i18n("Subdevide mesh"),34);
+			pm->insertItem(QPixmap((const char**)triangle_xpm),i18n("Simplify mesh"),35);
+			pm->insertItem(QPixmap((const char**)triangle_xpm),i18n("Smooth mesh"),36);
 		break;
 	}
 
@@ -678,11 +997,19 @@ void	WorldTree::popupMenu(QListViewItem *li,int x,int y)
 			QString		s;
 
 			s = b->getName();
-			nd = new NameDialog(this,"Name");
+			nd = new NameDialog(this,i18n("Name"));
 			nd->setName(s);
 			nd->show();
 			if(nd->result())
-				rename(li,nd->getName());
+			{
+				s = nd->getName();
+				rename(li,s);
+				if(b->getType() == NUM_VIEW)
+				{
+					emit changedViewName();
+				}
+			}
+			
 			delete nd;
 		}
 		break;
@@ -717,30 +1044,20 @@ void	WorldTree::popupMenu(QListViewItem *li,int x,int y)
 			if(b->isFlag(base::HOLLOW_OFF))
 				b->setFlag(base::HOLLOW_ON,0);
 		break;
-		case 3: // dim
+		case 3: // transform
 		{
-			DimDialog	*dd;
-			Vector3		v0;
-			Vector3		v1;
-			Vector3		v2;
+			TransformDialog	*td;
+			Vector3		&v0 = ((transform*)b)->getVScale();
+			Vector3		&v1 = ((transform*)b)->getVRotate();
+			Vector3		&v2 = ((transform*)b)->getVTranslate();
 
-			v0 = ((dim*)b)->getVScale();
-			v1 = ((dim*)b)->getVRotate();
-			v2 = ((dim*)b)->getVTranslate();
-
-			dd = new DimDialog(this,"Dimensions");
-
-			dd->load(v0,v1,v2);
-
-			dd->show();
-			if(dd->result())
-			{
-				dd->save(v0,v1,v2);
-				((dim*)b)->getVScale() = v0;
-				((dim*)b)->getVRotate() = v1;
-				((dim*)b)->getVTranslate() = v2;
-			}
-			delete dd;
+			td = new TransformDialog(this,i18n("Tranformations"),&v0,&v1,&v2);
+                        	
+			connect(td,SIGNAL(changedData()),this,SIGNAL(redraw()));
+			
+			td->show();
+			
+			delete td;
 		}
 		break;
 		case 6: // reference
@@ -752,7 +1069,7 @@ void	WorldTree::popupMenu(QListViewItem *li,int x,int y)
 			if(((refptr*)b)->getPTR())
 				s = ((refptr*)b)->getPTR()->getName();
 
-			nd = new NameDialog(this,"Name of Referenceobject");
+			nd = new NameDialog(this,i18n("Name of Referenceobject"));
 			nd->setName(s);
 			nd->show();
 			if(nd->result())
@@ -781,7 +1098,7 @@ void	WorldTree::popupMenu(QListViewItem *li,int x,int y)
 		{
 			BlobDialog	*bd;
 
-			bd = new BlobDialog(this,"Blob");
+			bd = new BlobDialog(this,i18n("Blob"));
 			bd->setThresholdValue(((blob*)b)->thresholdValue());
 			bd->show();
 			if(bd->result())
@@ -812,11 +1129,11 @@ void	WorldTree::popupMenu(QListViewItem *li,int x,int y)
 			Vector3			color;
 			double			fadepower,fadedistance;
 
-			fadepower = ((lightpoint*)b)->getFadePower();
-			fadedistance = ((lightpoint*)b)->getFadeDistance();
-			color = ((lightpoint*)b)->getColor();
+			fadepower = ((lightpoint*)b)->fadePower();
+			fadedistance = ((lightpoint*)b)->fadeDistance();
+			color = ((lightpoint*)b)->color();
 
-			ld = new LightPointDialog(this,"LightPoint");
+			ld = new LightPointDialog(this,i18n("Lightpoint"));
 
 			ld->load(fadepower,fadedistance,color);
 
@@ -832,6 +1149,44 @@ void	WorldTree::popupMenu(QListViewItem *li,int x,int y)
 		}
 		break;
 		case 17: // spotlight
+		{
+			SpotLightDialog		*ld;
+			Vector3			color;
+			double			fadepower,fadedistance;
+			double			radius,falloff,tightness;
+			bool			mediainteraction;
+			bool			mediaattenuation;
+
+			radius = ((spotlight*)b)->getRadius() / PI * 180;
+			falloff = ((spotlight*)b)->getFalloff() / PI * 180;
+			tightness = ((spotlight*)b)->getTightness();
+			fadepower = ((spotlight*)b)->fadePower();
+			fadedistance = ((spotlight*)b)->fadeDistance();
+			color = ((spotlight*)b)->color();
+ 			mediainteraction = ((spotlight*)b)->mediaInteraction();
+ 			mediaattenuation = ((spotlight*)b)->mediaAttenuation();
+
+			ld = new SpotLightDialog(this,i18n("SpotLight"));
+
+			ld->load(radius,falloff,tightness,fadepower,fadedistance,color,mediainteraction,mediaattenuation);
+
+			ld->show();
+			if(ld->result())
+			{
+				ld->save(radius,falloff,tightness,fadepower,fadedistance,color,mediainteraction,mediaattenuation);
+				falloff = MIN(80,falloff);
+				radius = MIN(falloff,radius);
+				((spotlight*)b)->setRadius(radius / 180 * PI);
+				((spotlight*)b)->setFalloff(falloff / 180 * PI);
+				((spotlight*)b)->setTightness(tightness);
+				((spotlight*)b)->setFadePower(fadepower);
+				((spotlight*)b)->setFadeDistance(fadedistance);
+				((spotlight*)b)->setColor(color);
+ 				((spotlight*)b)->setMediaInteraction(mediainteraction);
+ 				((spotlight*)b)->setMediaAttenuation(mediaattenuation);
+			}
+			delete ld;
+		}
 		break;
 		case 18: // arealight
 		break;
@@ -847,7 +1202,7 @@ void	WorldTree::popupMenu(QListViewItem *li,int x,int y)
 			fontname = ((text*)b)->getFontName();
 			thickness = ((text*)b)->getThickness();
 
-			td = new TextDialog(this,"Text");
+			td = new TextDialog(this,i18n("Text"));
 
 			td->load(context,fontname,thickness,offset);
 
@@ -863,6 +1218,20 @@ void	WorldTree::popupMenu(QListViewItem *li,int x,int y)
 			delete td;
 		}
 		break;
+		case 38: // script
+		{
+			ScriptDialog	*sd;
+
+			sd = new ScriptDialog(this,i18n("Script"),(script*)b);
+
+			sd->show();
+			if(sd->result())
+			{
+				sd->saveText();
+			}
+			delete sd;
+		}
+		break;
 		case 27: // Edit Animscript Main
 		case 28: // Edit Animscript
 			animScript(li);
@@ -875,7 +1244,7 @@ void	WorldTree::popupMenu(QListViewItem *li,int x,int y)
 			al = ((world*)b)->getAmbientLight();
 			bc = ((world*)b)->getBackgroundColor();
 
-			wd = new WorldDialog(this,"World");
+			wd = new WorldDialog(this,i18n("World"));
 
 			wd->load(al,bc);
 
@@ -914,7 +1283,7 @@ void	WorldTree::popupMenu(QListViewItem *li,int x,int y)
 				return;
 			}
 			
-			cd = new CameraDialog(this,"Select View");
+			cd = new CameraDialog(this,i18n("Select View"));
 
 			cd->setViewList(viewlist);
 
@@ -940,22 +1309,25 @@ void	WorldTree::popupMenu(QListViewItem *li,int x,int y)
 			HeightFieldDialog	*hd;
 			QString			str;
 			double			wl;
+			bool			sm;
 
 			str = ((heightfield*)b)->getFileName();
 			wl = ((heightfield*)b)->getWaterLevel();
+			sm = ((heightfield*)b)->getSmoothness();
 
-			hd = new HeightFieldDialog(this,"HieghtField");
+			hd = new HeightFieldDialog(this,i18n("Heightfield"));
 
-			hd->load(wl,str);
+			hd->load(wl,str,sm);
 
 			hd->show();
 			if(hd->result())
 			{
 				QImage		image;
 
-				hd->save(&wl,str);
+				hd->save(wl,str,sm);
 				((heightfield*)b)->setFileName(str);
 				((heightfield*)b)->setWaterLevel(wl);
+  				((heightfield*)b)->setSmoothness(sm);
 
 				if(image.load(str))
 				{
@@ -995,9 +1367,41 @@ void	WorldTree::popupMenu(QListViewItem *li,int x,int y)
 			delete hd;
 		}
 		break;
+		case 34: // Subdevide
+		{
+			((cmesh*)b)->subdevide();
+		}
+		break;
+		case 35: // Simplify
+		{
+			((cmesh*)b)->simplify(0.001);
+		}
+		break;
+		case 36: // Smooth
+		{
+			((cmesh*)b)->smooth();
+		}
+		break;
+		case 37: // Ear
+		{
+			EarDialog	*ed;
+			
+			ed = new EarDialog(this,i18n("Channelnumber"));
+			if(ed)
+			{
+				ed->setChannel(((ear*)b)->getChannel());
+				ed->show();
+				if(ed->result())
+					((ear*)b)->setChannel(ed->getChannel());
+				delete ed;
+			}
+		}
+		break;
 	}
 
 	delete pm;
+	
+	emit redraw();
 }
 
 void	WorldTree::animScript(QListViewItem *li)
@@ -1007,7 +1411,7 @@ void	WorldTree::animScript(QListViewItem *li)
 
 	if(!(b = worldptr->searchName((char*)li->text(0)))) return;
 
-	ad = new AnimDialog(this,"AnimScript",b);
+	ad = new AnimDialog(this,i18n("AnimScript"),b);
 	ad->exec();
 }
 
@@ -1037,6 +1441,9 @@ void	WorldTree::remove(QListViewItem *li)
 	delete li;
 
 	delete b;
+	
+	emit redraw();
+	emit changedModel();
 }
 
 void	WorldTree::cut(QListViewItem *li)
@@ -1055,6 +1462,8 @@ void	WorldTree::cut(QListViewItem *li)
 
 		if(capptr) delete capptr;
 		capptr = b;
+		emit redraw();
+		emit changedModel();
 	}	
 }
 
@@ -1097,6 +1506,8 @@ void	WorldTree::paste(QListViewItem *li)
 		else
 			num = 0;
 		createTree(child,li,num);
+		emit redraw();
+		emit changedModel();
 	}
 
 	capptr = 0;
@@ -1108,7 +1519,7 @@ void	WorldTree::getSelected(QListViewItem *li)
 
 	if(!(b = worldptr->searchName((char*)li->text(0)))) return;
 
-	emit setSelected(b);
+	emit changedSelected(b);
 }
 
 void	WorldTree::setTexture(texture *t)
@@ -1149,6 +1560,8 @@ void	WorldTree::setTexture(QString object,texture *t)
 			i->setText(1,t->getName());
 		else
 			i->setText(1,"");
+		emit redraw();
+		emit changedModel();
 	}
 }
 void	WorldTree::startDragCopy(QListViewItem *item,QWidget *dragsource)
@@ -1312,27 +1725,136 @@ void	WorldTree::changeTexture(texture *t,QListViewItem *i)
 	while(li)
 	{
 		b = worldptr->searchName(li->text(0));
-		bo = 0;
-		switch(b->getType())
-		{
-			case NUM_WORLD:
-			case NUM_CAMERA:
-			case NUM_DIM:
-			case NUM_BASE:
-			break;
-			default:
-				bo = (body*)b;
-		}
+                if (b) {
+			bo = 0;
+			switch(b->getType())
+			{
+				case NUM_WORLD:
+				case NUM_CAMERA:
+				case NUM_TRANSFORM:
+				case NUM_BASE:
+				break;
+				default:
+					bo = (body*)b;
+					emit redraw();
+					emit changedModel();
+			}
 
-		if(bo && bo->getTexture() == t)
-		{
-			li->setText(1,bo->getTexture()->getName());
+			if(bo && bo->getTexture() == t)
+			{
+				li->setText(1,bo->getTexture()->getName());
+			}
+			changeTexture(t,li->firstChild());
 		}
-		changeTexture(t,li->firstChild());
                 li = li->nextSibling();
 	}
 }
 
+void	WorldTree::lower()
+{
+	QListViewItem	*li,*lit,*pit;
+        base		*b,*p;
 
+	li = currentItem();	
+	if(li)
+	{
+		b = worldptr->searchName(li->text(0));
+		if(b)
+		{
+			p = b->getParent();
+			if(p)
+			{
+				if(p->lower(b) == 0)
+				{
+					QString		i;
+					base		**children = 0;
+					int		t;
+					
+					if((lit = li->itemBelow()))
+					{				
+						pit = li->parent();
+						empty(pit);
+						p->getChildren(&children);
+						if(!children)	return;
+
+						t = 0;
+						while(children[t])
+						{
+							createTree(children[t],pit,t);
+							t++;
+						}
+						if(children)
+							free(children);
+					}
+					emit redraw();
+					emit changedModel();
+				}
+			}
+		}
+	}
+}
+
+void	WorldTree::upper()
+{
+	QListViewItem	*li,*lit,*pit;
+        base		*b,*p;
+
+	li = currentItem();	
+	if(li)
+	{
+		b = worldptr->searchName(li->text(0));
+		if(b)
+		{
+			p = b->getParent();
+			if(p)
+			{
+				if(p->upper(b) == 0)
+				{
+					QString		i;
+					base		**children = 0;
+					int		t;
+					
+					if((lit = li->itemAbove()))
+					{				
+						pit = li->parent();
+						empty(pit);
+						p->getChildren(&children);
+						if(!children)	return;
+
+						t = 0;
+						while(children[t])
+						{
+							createTree(children[t],pit,t);
+							t++;
+						}
+						if(children)
+							free(children);
+					}
+					emit redraw();
+					emit changedModel();
+				}
+			}
+		}
+	}
+}
+
+void	WorldTree::updateViews()
+{
+	int		t;
+	TreeListItem	*vi;
+	view		*v;
+
+	if(!worldptr) return;
+
+	viewlistitem->clear();
+	
+	for(t = 0;t < worldptr->getNumViews();t++)
+	{
+		v = worldptr->getView(t);
+		vi = new TreeListItem(viewlistitem,v->getName(),20);
+		vi->setHeight(20);
+		vi->setPixmap(0,*pixmap[NUM_VIEW]);
+	}
+}
 
 
