@@ -29,6 +29,9 @@
 
 #include <stdlib.h>
 
+#include <misc.h>
+#include <evaluate.h>
+
 Preferences::Preferences()
 {
 	QColor			backcolor, drawcolor, selcolor, boxcolor;
@@ -36,7 +39,7 @@ Preferences::Preferences()
 	QColor			xaxiscolor,yaxiscolor,zaxiscolor;
 	QColor			letterxcolor,letterycolor,letterzcolor;
 	QString			datapath;
-	extern KApplication	*k3deapp;
+	extern KApplication	*k4deapp;
 	int			i,n,check;
         QString			f,s;
 
@@ -52,11 +55,12 @@ Preferences::Preferences()
 	letterxcolor = QColor(128,0,0);
 	letterycolor = QColor(0,128,0);
 	letterzcolor = QColor(0,0,128);
+	
 
 	cfg = new KSimpleConfig(getPath("~/.k4de/k4derc"));
 	if(cfg == 0)
 	{
-		datapath = k3deapp->kde_datadir() + "/k4de/";
+		datapath = k4deapp->kde_datadir() + "/k4de/";
 		cfg = new KSimpleConfig(getPath(datapath + "k4derc"));		
 	}
 
@@ -98,7 +102,24 @@ Preferences::Preferences()
 	setGridXY(cfg->readBoolEntry("GridXY",0));
 	setGridXZ(cfg->readBoolEntry("GridXZ",1));
 	setGridZY(cfg->readBoolEntry("GridZY",0));
-	
+
+
+	cfg->setGroup("Cluster");	
+
+	setShowClusterStatistics(cfg->readBoolEntry("ShowStatistics",true) );
+
+	clusterHosts.clear();
+
+	n=cfg->readNumEntry("ClusterNum",0);
+
+	for(i = 0;i <n;i++)
+	{
+		s.sprintf("ClusterHost%i",i);
+		f = cfg->readEntry(s,"");
+		if(f == "") continue;
+		setClusterHost(f);
+	}
+
 	
 	cfg->setGroup("Renderer");
 
@@ -122,6 +143,7 @@ Preferences::Preferences()
 
 	cfg->setGroup("Dialogs");
 
+	setMainWindowSize(cfg->readNumEntry("MainWindowWidth",700),cfg->readNumEntry("MainWindowHeight",500));
 	setAnimEditorSize(cfg->readNumEntry("AnimEditorWidth",400),cfg->readNumEntry("AnimEditorHeight",300));
 	
 	cfg->setGroup("Textures");
@@ -129,7 +151,7 @@ Preferences::Preferences()
 
 	cfg->setGroup("Menus");
 
-	setMaxNumRecentFiles((n = cfg->readNumEntry("MaxNumRecent",8)));
+    	setMaxNumRecentFiles((n = cfg->readNumEntry("MaxNumRecent",8)));	
 	setCheckExistance((check = cfg->readNumEntry("CheckExistance",0)));
 
 	recentfiles.clear();
@@ -137,11 +159,38 @@ Preferences::Preferences()
 	{
 		s.sprintf("RecentFile%i",i);
 		f = cfg->readEntry(s,"");
-		if(f == ""/* || check && !existFile(f)*/)
-			continue;
+		if(f == "") continue;
+
 		setRecentFile(f);
 	}
 	
+	
+	cfg->setGroup("Tools");
+
+	recursivlabels.clear();
+	recursivfunctions.clear();
+
+	i = cfg->readListEntry("RecursivTreeLabels",recursivlabels,'@');
+	n = cfg->readListEntry("RecursivTreeFunctions",recursivfunctions,'@');
+	
+	num_recursiventries = i;
+	
+#ifdef DEBUG
+	printf("labels (%i) and functions (%i)\n",i,n);
+#endif
+	if(i != n)
+	{
+#ifdef DEBUG
+		printf("Error in configfile: entrydifference between labels (%i) and functions (%i)\n",i,n);
+#endif
+		num_recursiventries = MIN(i,n);
+	}
+		
+		
+	cfg->setGroup("Misc");
+	setAutoSavePicture(cfg->readBoolEntry("AutoSavePicture",0));
+	setTranslatePOV(cfg->readBoolEntry("TranslatePOV",0));
+	
 	delete cfg;
 	cfg = 0;
 }
@@ -152,7 +201,7 @@ Preferences::~Preferences()
 	{
 		delete cfg;
 	}
-	save();
+	// save();
 }
 
 
@@ -214,6 +263,22 @@ double	Preferences::getAspectRatio()
 	return ar;
 }
 
+int	Preferences::getMainWindowWidth()
+{
+	return mainwin_width;
+}
+
+int	Preferences::getMainWindowHeight()
+{
+	return mainwin_height;
+}
+
+void	Preferences::setMainWindowSize(int w,int h)
+{
+	mainwin_width = w;
+	mainwin_height = h;
+}
+
 int	Preferences::getAnimEditorWidth()
 {
 	return animeditor_width;
@@ -759,12 +824,140 @@ void	Preferences::setRecentFile(QString f)
 	recentfiles.insert(0,f);
 }
 
+void	Preferences::setShowClusterStatistics(bool s)
+{
+	showClusterStatistics=s;
+}
+bool	Preferences::getShowClusterStatistics()
+{
+	return showClusterStatistics;
+}
+
+int	Preferences::numClusterHosts()
+{
+	return clusterHosts.count();
+}
+
+QString	Preferences::recentClusterHost(int n)
+{
+	QString		f,s;
+
+	if(n < 0 || n > numClusterHosts())
+		return f;
+
+	if(clusterHosts.at(n) != 0)
+		f = clusterHosts.at(n);
+				
+	return f;
+
+}
+void	Preferences::setClusterHost(QString f)
+{
+	int		i;
+	
+	for(i = 0;i < numClusterHosts();i++)
+	{
+		if(clusterHosts.at(i) == f)
+			return;
+	}
+	
+	clusterHosts.insert(0,f);
+
+}
+
+void 	Preferences::deleteClusterHostList()
+{
+	clusterHosts.clear();
+}
+
+void	Preferences::setAutoSavePicture(bool b)
+{
+	autosaverendered = b;
+}
+
+bool	Preferences::getAutoSavePicture()
+{
+	return autosaverendered;
+}
+
+int	Preferences::getRecursivTreeNumEntries()
+{
+	return num_recursiventries;
+}
+
+QString	Preferences::getRecursivTreeLabel(int n)
+{
+	return recursivlabels.at(n);
+}
+
+QString	Preferences::getRecursivTreeFunction(int n)
+{
+	return recursivfunctions.at(n);
+}
+
+void	Preferences::setRecursivTree(QString label,QString function)
+{
+	int		i;
+	QStrList	sll,slf;
+	
+	for(i = 0;i < num_recursiventries;i++)
+	{
+		if(getRecursivTreeLabel(i) != label)
+		{
+			sll.append(getRecursivTreeLabel(i));
+			slf.append(getRecursivTreeFunction(i));
+		}
+	}
+	sll.append(label);		
+	slf.append(function);
+	
+	recursivlabels = sll;
+	recursivfunctions = slf;
+	
+	num_recursiventries = MIN(recursivlabels.count(),recursivfunctions.count());
+	
+#ifdef DEBUG
+	printf("%i  %i  %i\n",num_recursiventries,recursivlabels.count(),recursivfunctions.count());
+#endif		
+}
+
+void	Preferences::deleteRecursivTree(QString label)
+{
+	int		i;
+	QStrList	sll,slf;
+	
+	for(i = 0;i < num_recursiventries;i++)
+	{
+		if(getRecursivTreeLabel(i) != label)
+		{
+			sll.append(getRecursivTreeLabel(i));
+			slf.append(getRecursivTreeFunction(i));
+		}
+	}
+	
+	recursivlabels = sll;
+	recursivfunctions = slf;		
+	
+	num_recursiventries = MIN(recursivlabels.count(),recursivfunctions.count());		
+}
+
+void	Preferences::setTranslatePOV(bool b)
+{
+	translate_pov = b;
+}
+
+bool	Preferences::getTranslatePOV()
+{
+	return translate_pov;
+}
+
 void Preferences::save()
 {
 	int		i;
 	QString		s;
 	
-	cfg = new KSimpleConfig(getPath("~/.k4de/k4derc"));
+	if(cfg == 0)
+		cfg = new KSimpleConfig(getPath("~/.k4de/k4derc"));
 	if(cfg != 0)
 	{
 		cfg->setGroup("Paths");
@@ -837,12 +1030,25 @@ void Preferences::save()
 		cfg->writeEntry("ResolutionHeight",getResolutionHeight());
 	
 		cfg->setGroup("Dialogs");
+		cfg->writeEntry("MainWindowWidth",getMainWindowWidth());
+		cfg->writeEntry("MainWindowHeight",getMainWindowHeight());
+//		printf("I wrote (%i,%i)\n",getMainWindowWidth(),getMainWindowHeight());
 		cfg->writeEntry("AnimEditorWidth",getAnimEditorWidth());
 		cfg->writeEntry("AnimEditorHeight",getAnimEditorHeight());
 		
 		cfg->setGroup("Textures");
 		cfg->writeEntry("DefaultTextureFile",getDefaultTextureFile());
 
+		cfg->setGroup("Cluster");	
+
+		cfg->writeEntry("ShowStatistics",getShowClusterStatistics() );
+		cfg->writeEntry("ClusterNum",numClusterHosts());
+		for(i = 0;i < numClusterHosts();i++)
+		{
+			s.sprintf("ClusterHost%i",i);
+			cfg->writeEntry(s,clusterHosts.at(i));
+		}
+
  		cfg->setGroup("Menus");
 		cfg->writeEntry("MaxNumRecentFiles",maxNumRecentFiles());
 		cfg->writeEntry("CheckExistance",checkExistance());
@@ -852,6 +1058,17 @@ void Preferences::save()
 			s.sprintf("RecentFile%i",i);
 			cfg->writeEntry(s,recentfiles.at(i));
 		}
+		cfg->setGroup("Misc");
+		cfg->writeEntry("AutoSavePicture",getAutoSavePicture());
+		cfg->writeEntry("TranslatePOV",getTranslatePOV());
+		
+		
+		cfg->setGroup("Tools");
+		cfg->writeEntry("RecursivTreeLabels",recursivlabels,'@');
+		cfg->writeEntry("RecursivTreeFunctions",recursivfunctions,'@');
+		
+		
+		cfg->sync();
 		
 		delete cfg;
 		cfg = 0;
@@ -882,6 +1099,18 @@ QString	Preferences::getPath(QString p)
 	return path;
 }
 
+QString	Preferences::povI18n(const char *t)
+{
+	QString		qstr;
+	
+	if(getTranslatePOV())
+		qstr = i18n(t);
+	else
+		qstr = t;
+		
+	return qstr;
+}
+
 
 
 
