@@ -319,7 +319,7 @@ void	Matrix44::unify()
 	}
 }
 
-Matrix44	&Matrix44::scaleVector(Vector3 &v)
+Matrix44	&Matrix44::scaleVector(Vector3 v)
 {
 	this->unify();
 	elements[0][0] = v[0];
@@ -329,7 +329,7 @@ Matrix44	&Matrix44::scaleVector(Vector3 &v)
 	return *this;
 }
 
-Matrix44	&Matrix44::transposeVector(Vector3 &v)
+Matrix44	&Matrix44::transposeVector(Vector3 v)
 {
 	this->unify();
 	elements[0][3] = v[0];
@@ -339,7 +339,17 @@ Matrix44	&Matrix44::transposeVector(Vector3 &v)
 	return *this;
 }
 
-Matrix44	&Matrix44::rotateVector(Vector3 &v)
+Matrix44	&Matrix44::translateVector(Vector3 v)
+{
+	this->unify();
+	elements[0][3] = v[0];
+	elements[1][3] = v[1];
+	elements[2][3] = v[2];
+
+	return *this;
+}
+
+Matrix44	&Matrix44::rotateVector(Vector3 v)
 {
 	Matrix44	h;
 	double		cx,sx,cy,sy,cz,sz;
@@ -811,29 +821,44 @@ double	Vector3::scalarprod(const Vector3 &v)
 		 + elements[2] * v.elements[2];
 }
 
+// distance point-point
 double	Vector3::distance(const Vector3 &v1)
 {
 	return ((*this - v1).length());
 }
 
+// distance point-line(point,point)
 double	Vector3::distance(const Vector3 &v1,const Vector3 &v2)
 {
-	Vector3		n;
+/*	Vector3		n;
 
 	n = ((v1 - v2) * (v1 - *this));
 	n.normalize();
 
 	return distance(v1,v2,v2 + n);
+*/
+	Vector3		a,u,p;
+	
+	a = v1;
+	u = v2 - v1;
+	p = *this;
+	
+	u.normalize();
+	
+	return (u * (p - a)).length();
 }
 
+// distance point-plane(point,point,point)
 double	Vector3::distance(const Vector3 &v1,const Vector3 &v2,const Vector3 &v3)
 {
-	Vector3		n;
+	Vector3		n,v;
 
 	n = ((v1 - v2) * (v1 - v3));
 	n.normalize();
 
-	return (n * (*this - v1)).length();
+	v = *this - v1;
+	
+	return fabs(n.scalarprod(v));
 }
 
 
